from __future__ import print_function
import os
import traceback
from collections import OrderedDict
import wx
import wx.dataview as dv
import wx.propgrid as pg
import six
import numpy as np
import time
import weakref
import traceback
try:
    #  for standalone testing (when running python hdf_export_window.py)
    from ifigure.utils.hdf_data_export import build_data,  hdf_data_export, set_all_properties_all, select_unique_properties_all
except:
    pass
import ifigure.events

from ifigure.utils.wx3to4 import PyDeadObjectError, menu_Append
'''
   helper window for HDF export

   HdfExportWindow(parent = <parent window>,
                   page = <figpage objct>)
'''
debug = False


class ObjectData(object):
    def __init__(self, data):
        self.data = data

    def GetData(self):
        return self.data

    def __repr__(self):
        return self.data.__repr__()


class HDFDataModel(dv.PyDataViewModel):
    def __init__(self, **kwargs):
        dv.PyDataViewModel.__init__(self)
        self.dataset = kwargs.pop('dataset', None)
        self.metadata = kwargs.pop('metadata', OrderedDict())
        self.export_flag = kwargs.pop('export_flag')
        # self.objmapper.UseWeakRefs(True)
        self.objs = []
        self.labels = {}

    def getobj(self, labels):
        if labels in self.labels:
            return self.labels[labels], True
        else:
            obj = ObjectData(labels)
            self.labels[labels] = obj
            return obj, False

    def GetChildren(self, parent, children):
        if debug:
            print('GetChildren')
        if not parent:
            num = 0
            for name in six.iterkeys(self.dataset):
                labels = (name,)
                obj, exist = self.getobj(labels)
                # if not exist:
                children.append(self.ObjectToItem(obj))
                if not labels in self.export_flag:
                    self.export_flag[labels] = True
                self.objs.append(obj)
                num = num + 1
            return num
        labels = self.ItemToObject(parent).GetData()
        p = self.dataset[labels[0]]
        for l in labels[1:]:
            p = p[l]
        if isinstance(p, dict):
            num = 0
            for newlabel in sorted(p.keys()):
                x = list(labels)
                x.append(newlabel)
                labels2 = tuple(x)
                obj, exist = self.getobj(labels2)
                # if not exist:
                children.append(self.ObjectToItem(obj))
                if not labels2 in self.export_flag:
                    self.export_flag[labels2] = self.export_flag[labels]
                self.objs.append(obj)
                num = num + 1
            return num
        return 0

    def GetParent(self, item):
        if debug:
            print('GetParent')
        if not item:
            return dv.NullDataViewItem

        ret = dv.NullDataViewItem
        obj = self.ItemToObject(item)
        labels = obj.data
        if len(labels) == 1:
            pass
        else:
            labels = obj.GetData()
            if labels[:-1] in self.labels:
                obj = self.labels[labels[:-1]]
                ret = self.ObjectToItem(obj)
        return ret

    def GetColumnCount(self):
        return 5

    def GetColumnType(self, col):
        mapper = {0: 'string',
                  1: 'bool',
                  2: 'string',
                  3: 'string',
                  4: 'string', }
        return mapper[col]

    def IsContainer(self, item):
        # The hidden root is a container
        if not item:
            return True
        # and if it is dict, the objects are containers
        labels = self.ItemToObject(item).GetData()
        p = self.dataset
        for l in labels:
            p = p[l]
        if isinstance(p, dict):
            return True
        # but everything else are not
        return False

    def GetValue(self, item, col):
        if debug:
            print('GetValue')
        labels = self.ItemToObject(item).GetData()
        p = self.dataset
        for l in labels:
            p = p[l]

        if col == 0:
            v = labels[-1]
            ret = v
        elif col == 1:
            ret = self.export_flag[labels]
        elif col == 2:
            if self.IsContainer(item):
                ret = ''
            else:
                ret = str(p)
                if len(ret) > 50:
                    ret = ret[:50] + '...'
        elif col == 3:
            if self.IsContainer(item):
                ret = ''
            else:
                if hasattr(p, 'shape'):
                    ret = str(p.shape)
                else:
                    ret = ''
        elif col == 4:
            p = 0
            d = self.metadata
            while labels[p] in d:
                d = d[labels[p]]
                dd = OrderedDict()
                for key in six.iterkeys(d):
                    if not isinstance(d[key], dict):
                        dd[key] = d[key]
                ret = str(dd)
                p = p+1
                if len(labels) <= p:
                    break
            else:
                ret = ''
            if self.IsContainer(item) and len(labels) != 1:
                ret = ''
            if ret.startswith('OrderedDict'):
                ret = ret[13:-2]  # tweak a visual ;D
        else:
            raise RuntimeError("unknown col")
        return ret

    def GetAttr(self, item, col, attr):
        labels = self.ItemToObject(item).GetData()
        if col == 0:
            attr.SetColour('blue')
            return True
        else:
            return False

    def SetValue(self, value, item, col):
        if debug:
            print('SetValue')
        labels = self.ItemToObject(item).GetData()
        p = self.dataset
        for l in labels:
            p = p[l]
        if col == 1:
            self.export_flag[labels] = value

            if value:
                for l2 in self.labels:
                    if len(l2) >= len(labels):
                        continue
                    if l2 == labels[:len(l2)]:
                        self.export_flag[l2] = value
                        obj = self.labels[l2]
                        item2 = self.ObjectToItem(obj)
                        self.ValueChanged(item2, col)
            if self.IsContainer(item):
                labels = self.ItemToObject(item).GetData()
                for l2 in self.labels:
                    if len(l2) <= len(labels):
                        continue
                    if l2[:len(labels)] == labels:
                        self.export_flag[l2] = value
                        obj = self.labels[l2]
                        item2 = self.ObjectToItem(obj)
                        self.ValueChanged(item2, col)

    def HasContainerColumns(self, item):
        return True


class HdfExportWindow(wx.Frame):
    def __init__(self, *args, **kargs):
        page = kargs.pop('page', None)
        parent = kargs.pop('parent', None)
        title = kargs.pop('title', 'HDF export')
        path = kargs.pop('path', os.path.join(os.getcwd(), 'data.hdf'))
        dataset = kargs.pop('dataset', None)
        metadata = kargs.pop('metadata', OrderedDict())
        export_flag = kargs.pop('export_flag', {})
        if dataset is None:
            if page is None:
                return
            dataset, metadata, export_flag = build_data(page,
                                                        verbose=False,
                                                        metadata=metadata,
                                                        export_flag=export_flag)
        style = (wx.CAPTION | wx.CLOSE_BOX | wx.MINIMIZE_BOX |
                 wx.RESIZE_BORDER)
        if parent is not None:
            style = style | wx.FRAME_FLOAT_ON_PARENT

        wx.Frame.__init__(self, *args,
                          style=style,
                          title=title, parent=parent,
                          size=(400, 500))

        self.SetSizer(wx.BoxSizer(wx.VERTICAL))
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        self.GetSizer().Add(hsizer, 1, wx.EXPAND, 0)
        panel = wx.Panel(self)
        hsizer.Add(panel, 1, wx.EXPAND)

        panel.SetSizer(wx.BoxSizer(wx.VERTICAL))
        sizer = panel.GetSizer()
        sizer_h = wx.BoxSizer(wx.HORIZONTAL)

        wildcard = "HDF(hdf)|*.hdf|Any|*.*"
        self.filepicker = wx.FilePickerCtrl(panel,
                                            style=wx.FLP_SAVE | wx.FLP_USE_TEXTCTRL,
                                            path=path,
                                            wildcard=wildcard)
        self.sp = wx.SplitterWindow(panel, wx.ID_ANY,
                                    style=wx.SP_NOBORDER | wx.SP_LIVE_UPDATE | wx.SP_3DSASH)
        self.dataviewCtrl = dv.DataViewCtrl(self.sp,
                                            style=(wx.BORDER_THEME
                                                   | dv.DV_ROW_LINES
                                                   | dv.DV_VERT_RULES))

        self.model = HDFDataModel(export_flag=export_flag,
                                  dataset=dataset,
                                  metadata=metadata)
        self.dataviewCtrl.AssociateModel(self.model)

        self.tr = tr = dv.DataViewTextRenderer()
        c0 = dv.DataViewColumn("name",   # title
                               tr,        # renderer
                               0,         # data model column
                               width=200)
        self.dataviewCtrl.AppendColumn(c0)

#        self.dataviewCtrl.AppendTextColumn('name', 0,
#                                           width = 150,
#                                           mode=dv.DATAVIEW_CELL_ACTIVATABLE)
        c1 = self.dataviewCtrl.AppendToggleColumn('export', 1,
                                                  width=70,
                                                  mode=dv.DATAVIEW_CELL_ACTIVATABLE)
        c2 = self.dataviewCtrl.AppendTextColumn('value', 2, width=100,
                                                mode=dv.DATAVIEW_CELL_ACTIVATABLE)
        c3 = self.dataviewCtrl.AppendTextColumn('shape', 3,
                                                mode=dv.DATAVIEW_CELL_ACTIVATABLE)
        #c4 = self.dataviewCtrl.AppendTextColumn('metadata', 4,
        #                                        mode=dv.DATAVIEW_CELL_ACTIVATABLE)
        for c in self.dataviewCtrl.Columns:
            c.Sortable = True
            c.Reorderable = True
        c1.Alignment = wx.ALIGN_CENTER

        #from ifigure.widgets.var_viewerg2 import _PropertyGrid

        self.grid = pg.PropertyGrid(self.sp)

        #self.btn_load = wx.Button(self, label = 'Load')
        self.choices = ['Options...', 'No Property',
                        'Minimum properties',
                        'All properties']
        self.cb = wx.ComboBox(panel, wx.ID_ANY,
                              style=wx.TE_PROCESS_ENTER | wx.CB_READONLY,
                              choices=self.choices)
        self.cb.SetValue(self.choices[0])
        self.btn_export = wx.Button(panel, label='Export...')

        sizer.Add(self.filepicker, 0, wx.EXPAND | wx.ALL, 1)
        sizer.Add(self.sp, 1, wx.EXPAND | wx.ALL, 5)
        sizer.Add(sizer_h, 0, wx.EXPAND | wx.ALL, 5)

        #sizer_h.Add(self.btn_load, 0,   wx.EXPAND|wx.ALL, 1)
        sizer_h.Add(self.cb, 0,   wx.EXPAND | wx.ALL, 1)
        sizer_h.AddStretchSpacer(prop=1)
        sizer_h.Add(self.btn_export, 0, wx.EXPAND | wx.ALL, 1)

        self.Layout()
        self.Show()

        self.sp.SetMinimumPaneSize(30)
        self.sp.SplitHorizontally(self.dataviewCtrl, self.grid)
        self.sp.SetSashPosition(300)
#        self.Bind(dv.EVT_DATAVIEW_ITEM_VALUE_CHANGED,
#                  self.onDataChanged, self.dataviewCtrl)
        self.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED,
                  self.onSelChanged, self.dataviewCtrl)

        # self.Bind(dv.EVT_DATAVIEW_ITEM_COLLAPSING,
        #          self.onCollapsing, self.dataviewCtrl)

        self.grid.Bind(pg.EVT_PG_CHANGED,
                       self.onPGChanged,  self.grid)
        self.Bind(wx.EVT_BUTTON, self.onExport, self.btn_export)
        self.Bind(wx.EVT_COMBOBOX, self.onCBHit, self.cb)
        self.Bind(wx.EVT_CLOSE, self.onWindowClose)

        self.page = page
        wx.GetApp().TopWindow.hdf_export_window = self

    def onCBHit(self, evt):
        flags = self.model.export_flag
        dataset = self.model.dataset
        idx = self.choices.index(str(self.cb.GetValue()))

        self.cb.SetValue(self.choices[0])
        if idx == 1:
            set_all_properties_all(flags, False)
        elif idx == 2:
            select_unique_properties_all(self.page, dataset, flags)
        elif idx == 3:
            set_all_properties_all(flags, True)
        else:
            pass
        self.Refresh()
        evt.Skip()

    def onTD_Selection(self, evt):
        w = evt.GetEventObject()
        if w is self:
            return

        td = evt.GetTreeDict()
        name = self.page.name + '.' + '.'.join(self.page.get_td_path(td)[1:])

        item = self.dataviewCtrl.GetSelection()
        try:
            clabels = self.model.ItemToObject(item).GetData()
        except:
            clabels = (self.page.name, )

        if clabels[0] == name:
            return
        if (name,) in self.model.labels:
            obj, flag = self.model.getobj((name,))
            item = self.model.ObjectToItem(obj)
            self.dataviewCtrl.Select(item)

    def onSelChanged(self, evt):
        # print('onSelChanged')
        item = self.dataviewCtrl.GetSelection()
        try:
            labels = self.model.ItemToObject(item).GetData()
        except:
            self.grid.Clear()
            self.dataviewCtrl.UnselectAll()
            return
        metadata = self.model.metadata
        self.grid_target = None
        p = 0
        d = metadata
        dd = OrderedDict()
        while labels[p] in d:
            d = d[labels[p]]
            dd = OrderedDict()
            for key in six.iterkeys(d):
                if not isinstance(d[key], dict):
                    dd[key] = d[key]
            ret = dd, d
            p = p+1
            if len(labels) <= p:
                break
        else:
            ret = None
        if self.model.IsContainer(item) and len(labels) != 1:
            ret = None
        self.grid.Clear()
        for key in six.iterkeys(dd):
            prop = pg.StringProperty(str(key), value=str(dd[key]))
            self.grid.Append(prop)
        self.grid_target = d

        names = labels[0].split('.')
        if len(names) > 1:
            figobj = self.page
            for name in names[1:]:
                figobj = figobj.get_child(name=name)
            if len(figobj._artists) > 0:
                sel = [weakref.ref(figobj._artists[0])]
                ifigure.events.SendSelectionEvent(figobj, self, sel)
        evt.Skip()

    def onPGChanged(self, evt):
        prop = self.grid.GetSelection()
        if self.grid_target is not None:
            self.grid_target[prop.GetLabel()] = str(prop.GetValue())
        evt.Skip()

    def onExport(self, evt):
        import ifigure.widgets.dialog as dialog
        flags = self.model.export_flag
        dataset = self.model.dataset
        metadata = self.model.metadata
        path = self.filepicker.GetPath()
        try:
            hdf_data_export(data=dataset,
                            metadata=metadata,
                            export_flag=flags,
                            filename=path,
                            verbose=True)
            print('HDF export finished : '+path)
        except:
            dialog.showtraceback(parent=self,
                                 txt='Failed to export HDF',
                                 title='Error during HDF export',
                                 traceback=traceback.format_exc())
        dialog.message(parent=self, style=0,
                       message='Export finished')
        # self.Close()
        evt.Skip()

    def onWindowClose(self, evt):
        wx.GetApp().TopWindow.hdf_export_window = None
        evt.Skip()


if __name__ == '__main__':
    '''
    sample data for testing
    '''
    import sys
    import os
    dataset = OrderedDict()
    dataset['page1'] = {}
    dataset['page1']['property'] = {}
    dataset['page1']['property']['bgcolor'] = 'red'
    dataset['page1']['property']['frame'] = 'False'
    dataset['page1.axes1'] = {}
    dataset['page1.axes1']['property'] = {}
    dataset['page1.axes1']['property']['xlabel'] = 'time'
    dataset['page1.axes1.plot1'] = {}
    dataset['page1.axes1.plot1']['data1'] = {}
    dataset['page1.axes1.plot1']['data1']['xdata'] = np.arange(30)
    dataset['page1.axes1.plot1']['data1']['ydata'] = np.arange(30)

    app = wx.App(redirect=False)
    w = HdfExportWindow(dataset=dataset)
    w.Show()
    app.MainLoop()



import sys
import socket
import shlex
import threading
import os
import binascii
import subprocess
import time
import tempfile
from six.moves import socketserver
import traceback
import ifigure.utils.pickle_wrapper as pickle
from ifigure.utils.pickled_pipe import PickledPipe


def pick_unused_port():
    '''
    a routeint to le a system to pick an unused port.
    this recipe was found in active state web site
    '''
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    addr, port = s.getsockname()
    s.close()
    return port


class Client(object):
    def __init__(self, *args, **kargs):
        self.sock = None
        self.rfile = None
        object.__init__(self, *args, **kargs)

    def make_connection(self, ip, port):
        if self.sock is None:
            try:
                self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.sock.connect((ip, port))
                self.rfile = self.sock.makefile('r')
            except:
                self.sock = None
                self.rfile = None

    def process(self, ip, port, message):
        if len(message) == 0:
            return {}

        v = {}

        try:
            if self.sock is None:
                self.make_connection(ip, port)
            msr = pickle.dumps(message)
            msbin = binascii.b2a_hex(msr)
            self.sock.sendall(msbin+'\n')
#
            response = self.rfile.readline().strip()
#            self.rfile.close()
            try:
                v = pickle.loads(binascii.a2b_hex(response))
            except EOFError:
                v = {'client_status': 'error in loading data'}
        except:
            v = {'client_status': 'error in sending data'}
        finally:
            pass
#            self.sock.close()
#            self.sock = None
        v['message'] = [message]
        return v

    def finish_connection(self):
        if self.rfile is not None:
            self.rfile.close()
        if self.sock is not None:
            self.sock.close()
        self.sock = None
        self.rfile = None


class JobRunner(object):
    def __init__(self, host, port):
        self.p = None
        self.host = 'localhost'
        lport = self.make_ssh_connection(host, port)
        self.port = lport
        self.client = Client()
        self._shot = -1
        self._connection_param = ''
        self.gjob = None
        try:
            time.sleep(3)
            self.client.make_connection(self.host, self.port)
        except:
            pass
        self.g = {}

    def __del__(self):
        from signal import SIGTERM, SIGKILL
        if self.client is not None:
            self.client.finish_connection()
        self.client = None
        if self.p is not None:
            os.kill(self.p.pid, SIGKILL)

    def terminate(self):
        if self.client is not None:
            self.client.finish_connection()
        self.client = None
        if self.p is not None:
            self.p.kill()
        self.p == None

    def make_ssh_connection(self, host, port):
        lport = pick_unused_port()

        command = 'ssh -N  -L ' +  \
            str(lport)+':' + str(host)+':' + \
            str(port) + ' ' + str(host)

        self.p = subprocess.Popen(shlex.split(command))
        return lport

    def set_globaljob(self, job):
        self.gjob = job

    def run(self, jobset, skip_init):
        self.error = []
        jobs, names = jobset

        all_commands = []
        for k, job in enumerate(jobs):
            commands = []
            for j in job:
                if j.command != 'script_txt':
                    if j.command == 'init':
                        if skip_init:
                            continue
                        else:
                            j.command = 'value'
#                    command, command2 = self._job_command(j)
                    ccc = self._job_command(j)
                    commands.extend(ccc)
#                    if command != '':
#                       commands.append(command)
#                    if command2 != '':
#                       commands.append(command2)
            if len(commands) > 0:
                all_commands.append((names[k], commands))
#        print all_commands
        self.r = self.client.process(self.host, self.port, all_commands)
        for k, job in enumerate(jobs):
            for j in job:
                if j.command == 'script_txt':
                    self.r[names[k]] = self._run_script(j)
        self.r['error message'] = self.error
        return self.r

    def _job_command(self, job):
        com = job.command
#           command = ''
#           command2 = ''
        ccc = []
        if com == 'open':
            tree = job.params[0]
            shot = job.params[1]
#               command = "c"+str(tree)+","+ str(shot)
            ccc.append("c"+str(tree)+"," + str(shot))
            if shot != self._shot:
                expr = 'reset_private();reset_public();1'
#                   command2 = "v"+str(expr)
                ccc.append("v"+str(expr))
            if self._connection_flag:
                if self.gjob is not None:
                    expr = self.gjob.params[0]
                    ccc.append("v"+str(expr))
            self._connection_flag = True
            self._shot = shot
        elif com == 'defnode':
            node = job.params[0]
            if node.strip() != '':
                ccc.append("d"+str(node))
#               else:
#                   command = ''
        elif com == 'value':
            expr = job.params[0]
#               command = "v"+str(expr)
            ccc.append("v"+str(expr))
        elif com == 'valuesig':
            expr = job.params[0]
#               command = "v"+str(expr)
            ccc.append("u"+str(expr))
        elif com == 'dim_of':
            expr = job.params[0][2:]
            ccc.append("w"+str(expr))
        elif com == 'connection_mode':
            if self._connection_param != job.params[0]:
                #                  command = 's'+ job.params[0]
                ccc.append('s' + job.params[0])
                self._connection_param = job.params[0]
                self._connection_flag = True
        return ccc
#           return command, command2

    def _run_script(self, job):
        expr = job.params[0]
        try:
            code = compile(expr, '<string>', 'exec')
            g = globals()
            l = {}
            exec(code, self.g, self.r)
            return 'ok'
        except:
            self.error = ['Scrip Error', expr, sys.exc_info()[0]]
            return None



from __future__ import print_function

'''

   Export (partial) Figure Data to HDF file



'''
import traceback
import six
from collections import OrderedDict

from ifigure.widgets.artist_widgets import listparam, call_getter


def set_default_export_flag(obj, name, flag):
    from ifigure.mto.fig_page import FigPage
    from ifigure.mto.fig_axes import FigAxes

    name = (name, )
    if isinstance(obj, FigPage):
        flag[name] = True
        return
    if isinstance(obj, FigAxes):
        flag[name] = False
        return
    try:
        dd = obj.export()
    except NotImplementedError:
        flag[name] = False
        return
    flag[name] = True


def select_unique_properties(parent, dataset, flags):
    nb_done = []
    page = parent.get_figpage()
    for name, child in parent.get_children():
        nb = child.get_namebase()
        if nb in nb_done:
            continue
        nb_done.append(nb)
        objs = [child for name, child in parent.get_children()
                if child.get_namebase() == nb]
        names = [page.name + '.' + '.'.join(page.get_td_path(obj)[1:])
                 for obj in objs]
        if len(objs) == 1:
            labels = (names[0], 'property')
            flags[labels] = False
            for key in six.iterkeys(dataset[names[0]]['property']):
                labels = (names[0], 'property', key)
                flags[labels] = False
        props = [dataset[name]['property'] for name in names]
        pp = tuple()
        for p in props:
            pp = pp + tuple(p)
        for x in set(pp):
            value = not (len(set([p.get(x, None) for p in props])) == 1)
            if value:
                for name in names:
                    labels = (name, 'property')
                    flags[labels] = value
            for name in names:
                labels = (name, 'property', x)
                flags[labels] = value


def select_unique_properties_all(page, dataset, flags):
    for obj in page.walk_tree():
        if obj.num_child() > 0:
            select_unique_properties(obj, dataset, flags)
    for key in six.iterkeys(flags):
        if (key[0] == page.name and len(key) >= 2 and
                key[1] == 'property'):
            flags[key] = False


def set_all_properties_all(flags, value):
    keys = list(flags)
    for labels in keys:
        if len(labels) < 2:
            continue
        if labels[1] == 'property':
            flags[labels] = value
            
    keys = list(flags)
    for labels in keys:
        if not (labels[0], 'property') in flags:
            flags[(labels[0], 'property')] = value


def get_all_properties(obj):
    ret = obj.property_for_shell()
    tags = None
    if isinstance(ret, tuple):
        tab = ret[0]
        props = ret[1]
        if len(ret) == 3:
            tags = ret[2]
    else:
        tab = ['']
        props = [ret]
    if tags is None:
        tags = ['']*len(props)

    ll = []
    for tag, plist in zip(tags, props):
        for p in plist:
            if tag == '':
                ll.append(('',  listparam[p][6], listparam[p]))
            else:
                ll.append((tag, listparam[p][6], listparam[p]))

    props = {}
    for lll in ll:
        tag, name, lp = lll
        ret = [call_getter(a, lp, tab=tag) for a in obj._artists]
        if len(ret) == 1:
            ret = str(ret[0])
        else:
            ret = [str(x) for x in ret]
        if tag == '':
            props[name] = ret
        else:
            props['_'.join((tag, name))] = ret
    return props


def build_data(page, export_flag=None,
               metadata=None,  verbose=True):
    if export_flag is None:
        export_flag = {}
    if metadata is None:
        metadata = {}
    book = page.get_figbook()
    page.assign_default_file_metadata()
    dataset = OrderedDict()

    for obj in page.walk_tree():
        if obj is page:
            name = page.name
        else:
            name = page.name + '.' + '.'.join(page.get_td_path(obj)[1:])

        dataset[name] = {}
        set_default_export_flag(obj, name, export_flag)
        try:
            dd = obj.export()
        except NotImplementedError:
            if verbose:
                print(name + ' does not have data to export')
        except:
            print('Unexpected error')
            raise
        else:
            if len(dd) == 1:
                dataset[name]['data'] = dd[0]
            else:
                for kk,  ddd in enumerate(dd):
                    dataset[name]['data'+str(kk+1)] = ddd
            obj.assign_default_metadata()
            obj.update_data_metadata()
            if verbose:
                txt = ['member ' + str(i) + ' exprot ' + ','.join(list(d))
                       for i, d in enumerate(dd)]
                print(name + ' : ' + ','.join(txt))
        try:
            props = get_all_properties(obj)
        except:
            print('Unexpected error')
            raise
        else:
            dataset[name]['property'] = props
            if verbose:
                txt = '\n'.join([k + ':' + props[k] for k in props])
                print('property')
                print(txt)
        if obj.has_metadata():
            metadata[name] = obj.getvar('metadata')
    return dataset, metadata, export_flag


def hdf_data_export(page=None,
                    filename='data.hdf',
                    verbose=False,
                    dry_run=False,
                    data=None,
                    metadata=None,
                    export_flag=None):

    if export_flag is None:
        export_flag = {}
    if metadata is None:
        metadata = OrderedDict()
    if data is None:
        if page is None:
            print('Error: Specify either page object or data')
            return
        data, metadata, export_flag = build_data(page,
                                                 verbose=verbose,
                                                 metadata=metadata,
                                                 export_flag=export_flag)
    if dry_run:
        return

    import h5py
    #from netCDF4 import Dataset
    rootgrp = h5py.File(filename, 'w')
    #rootgrp = Dataset(filename, "w", format="NETCDF4")

    import time
    meta = metadata[list(metadata)[0]]
    meta['description'] = "Figure data exported from piScope"
    meta['date'] = time.ctime(time.time())
    for key in six.iterkeys(meta):
        rootgrp.attrs[key] = str(meta[key])
    metadata[list(metadata)[0]] = {}

    for key in six.iterkeys(data):
        labels = (key, )
        if (labels in export_flag and
                not export_flag[labels]):
            continue
        key_grp = rootgrp.create_group(key)

        data_keys = [k for k in data[key] if k.startswith('data')]
        for i, k in enumerate(data_keys):
            if len(data_keys) > 1:
                data_grp = key_grp.create_group(k)
            else:
                data_grp = key_grp
            ddd = data[key][k]
            for key2 in six.iterkeys(ddd):
                labels = (key, k, key2)
                # print 'checking ', labels, ddd.keys()
                if (labels in export_flag and
                        not export_flag[labels]):
                    continue

                do_complex = False
                if ddd[key2].dtype == complex:
                    do_complex = True
                try:
                    meta = metadata[key][k][key2]  # plot, data, xdata
                except:
                    meta = {}
                if do_complex:
                    cdata_grp = data_grp.create_group(key2 + '(Complex)')
                    dataset1 = cdata_grp.create_dataset(
                        'Real', data=ddd[key2].real)
                    dataset2 = cdata_grp.create_dataset(
                        'Imag', data=ddd[key2].imag)
                    for key3 in six.iterkeys(meta):
                        cdata_grp.attrs[key3] = str(meta[key3])
                    dataset1.attrs['comment'] = 'real part'
                    dataset2.attrs['comment'] = 'imaginary part'
                    dataset1.attrs['shape'] = str(ddd[key2].real.shape)
                    dataset2.attrs['shape'] = str(ddd[key2].imag.shape)
                else:
                    dataset = data_grp.create_dataset(key2, data=ddd[key2])
                    for key3 in six.iterkeys(meta):
                        dataset.attrs[key3] = str(meta[key3])
                    dataset.attrs['shape'] = str(ddd[key2].shape)
        try:
            meta = metadata[key]
        except:
            meta = {}
        for key3 in six.iterkeys(meta):
            if not isinstance(meta[key3], dict):
                key_grp.attrs[key3] = str(meta[key3])

        props = data[key]['property']
        for key2 in six.iterkeys(props):
            labels = (key, 'property', key2)
            if (labels in export_flag and
                    not export_flag[labels]):
                continue
            key_grp.attrs[key2] = str(props[key2])

    rootgrp.close()

    return data



from __future__ import print_function
import wx
from ifigure.utils.cbook import parseStr
from ifigure.mto.py_code import PyData
from ifigure.mto.py_code import PyCode
import os
import re
import string
import numpy
from collections import OrderedDict
import idlsave

######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.


#      Strong recommendation : make module "independent".

#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module
module_name = 'gfile_subs'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
module_evt_handler = [("Load File", "onLoadFile", True)]
#
#   A function called when py_module is initialized
#
module_init = None
######################################################


def expand_recarray(top, var):
    for name in var.dtype.names:
        print((name,  type(var.field(name))))

        if isinstance(var.field(name), numpy.recarray):
            child = PyData()
            top.add_child(name, child)
            expand_recarray(child, var.field(name))
        else:
            top.setvar(name, var.field(name))


def read_sav_file(this):
    file = this.getvar("filename")
    s = idlsave.read(file)
    for key in s:
        if isinstance(s[key], numpy.recarray):
            child = PyData()
            this.add_child(key, child)
            expand_recarray(child, s[key])
        else:
            this.setvar(key, s[key])


def onLoadFile(self_obj):
    open_dlg = wx.FileDialog(None, message="Select .sav file",
                             wildcard='*.*',
                             style=wx.FD_OPEN)
    if open_dlg.ShowModal() != wx.ID_OK:
        open_dlg.Destroy()
        return False
    file = open_dlg.GetPath()
    open_dlg.Destroy()
    for name, child in self_obj.get_children():
        child.destroy()
    self_obj.setvar("filename", file)
    read_sav_file(self_obj)


def onWriteFile(self_obj):
    pass



from __future__ import print_function
#
#  a script to produce icon images
#
from ifigure.ifigure_config import plinestyle_list
from ifigure.ifigure_config import linewidth_list
from ifigure.ifigure_config import linestyle_list, linestylenames
from ifigure.ifigure_config import marker_list, markernames
from ifigure.ifigure_config import color_list
from ifigure.ifigure_config import arrowstyle_list
from ifigure.utils.cbook import register_idl_colormaps
from ifigure.ifigure_config import colormap_list
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.transforms import Bbox
import base64
from ifigure.ifigure_config import isMPL2
import wx

b64encode = base64.urlsafe_b64encode

def encode(txt):
    return (b64encode(txt.encode())).decode()

##################################################
def generate_colormap():
    dpi = 72
    dpi2 = dpi

    F = plt.figure(num=None, figsize=(2.4, 0.3), dpi=dpi, facecolor='w')
    x = [0, 1, 1, 0, 0]
    y = [1, 1, 0, 0, 1]


    print('### generating colormap palette button ###')
    idl_names = register_idl_colormaps()

    a = np.linspace(0, 1, 256).reshape(1, -1)
    a = np.vstack([a]*20)

    maps = [colormap_list()[0]] + colormap_list() + idl_names
    for map in maps:
        ax = plt.subplot(111)
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.imshow(a, cmap=plt.get_cmap(map), origin='lower')
        filename = 'colormap_'+encode(map)+'.png'
        ed = ax.transAxes.transform([(0, 0), (1, 1)])
        bbox = Bbox.from_extents(
            ed[0, 0]/dpi, ed[0, 1]/dpi, ed[1, 0]/dpi, ed[1, 1]/dpi)
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)
    ##################################################

def generate_arrow():    
    dpi = 72
    dpi2 = dpi/3

    F = plt.figure(num=None, figsize=(0.6, 0.325), dpi=dpi, facecolor='w')
    ax = plt.subplot(111)
    x = [0, 1, 1, 0, 0]
    y = [1, 1, 0, 0, 1]

    ed = ax.transAxes.transform([(0, 0), (1, 1)])
    bbox = Bbox.from_extents(
        (ed[0, 0]+1)/dpi, (ed[0, 1]+1)/dpi, ed[1, 0]/dpi, (ed[1, 1]-2)/dpi)

    print('### generating arrow palette button ###')

    a = np.linspace(0, 1, 256).reshape(1, -1)
    a = np.vstack((a, a))

    for name, style in arrowstyle_list():
        ax.cla()
        ax.tick_params(length=0)
        ax.annotate('', [1, 0.5], xytext=[0, 0.5],
                    arrowprops=dict(arrowstyle=style))
        filename = 'arrow_'+encode(name.encode())+'.png'
        ed = ax.transAxes.transform([(0, 0), (1, 1)])
        bbox = Bbox.from_extents(
            ed[0, 0]/dpi, ed[0, 1]/dpi, ed[1, 0]/dpi, ed[1, 1]/dpi)
        plt.savefig(filename, dpi=dpi, format='png', bbox_inches=bbox)

    ##################################################
def generate_color():
    dpi = 72
    dpi2 = dpi/3

    F = plt.figure(num=None, figsize=(1.2, 0.75), dpi=dpi, facecolor='w')
    ax = plt.subplot(111)
    x = [0, 1, 1, 0, 0]
    y = [1, 1, 0, 0, 1]

    ed = ax.transAxes.transform([(0, 0), (1, 1)])
    bbox = Bbox.from_extents(ed[0, 0]/dpi, ed[0, 1]/dpi,
                             ed[1, 0]/dpi, ed[1, 1]/dpi)

    print('### generating color palette button ###')

    collist = color_list()
    for color in collist:
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.fill(x, y, color=color)
        if color == 'none':
            plt.plot([0, 1], [1, 0], color='red')
        filename = 'color_'+encode(color)+'.png'
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)

    ax.cla()
    ax.tick_params(length=0)
    ax.set_axis_off()
    ax.text(0.5, 0.2, '?', ha='center', fontsize=30)
    filename = 'color_'+encode('other')+'.png'
    plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)

def generate_symbol():
    dpi = 72
    dpi2 = dpi/3
    
    print('### generating symbol palette button ###')
    x = [0, 0.5, 1]
    y = [0.5, 0.5, 0.5]
    for marker, mname in zip(marker_list(), markernames):
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.plot(x, y, marker=marker, linestyle='None',
                 markevery=(1, 2), color='black', markersize=30)
        filename = 'marker_'+encode(str(mname))+'.png'
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)


def generat_linestyle():
    dpi = 72
    dpi2 = dpi/3
    
    print('### generating linestyle palette button ###')
    linestylelist = linestyle_list()

    x = [0, 0.5, 1]
    y = [0.5, 0.5, 0.5]
    for linestyle, lname in zip(linestylelist, linestylenames):
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.plot(x, y, linestyle=linestyle, color='black', linewidth=5)
        filename = 'linestyle_'+encode(str(lname))+'.png'
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)


def generat_linewidth():
    dpi = 72
    dpi2 = dpi

    F = plt.figure(num=None, figsize=(0.4, 0.25), dpi=dpi, facecolor='w')
    ax = plt.subplot(111)

    print('### generating linewidth palette button ###')
    linewidthlist = linewidth_list()
    linewidthlist = ['0.0'] + linewidth_list()


    x = [0, 0.5, 1]
    y = [0.5, 0.5, 0.5]
    ed = ax.transAxes.transform([(0, 0), (1, 1)])
    bbox = Bbox.from_extents(ed[0, 0]/dpi, ed[0, 1]/dpi,
                             ed[1, 0]/dpi, ed[1, 1]/dpi)
    for linewidth in linewidthlist:
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.plot(x, y, linewidth=linewidth, color='black')
        filename = 'linewidth_'+encode(str(linewidth))+'.png'
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)


##################################################
def generate_plinestyle():
    dpi = 72
    dpi2 = dpi*5 if isMPL2 else dpi

    F = plt.figure(num=None, figsize=(0.4, 0.25), dpi=dpi, facecolor='w')
    ax = plt.subplot(111)
    x = [0, 1, 1, 0, 0]
    y = [1, 1, 0, 0, 1]
    ed = ax.transAxes.transform([(0, 0), (1, 1)])
    bbox = Bbox.from_extents(ed[0, 0]/dpi, ed[0, 1]/dpi,
                             ed[1, 0]/dpi, ed[1, 1]/dpi)

    print('### generating patch linestyle palette button ###')
    plinestylelist = plinestyle_list()

    x = [0, 0.5, 1]
    y = [0.5, 0.5, 0.5]
    for plinestyle in plinestylelist:
        ax.cla()
        ax.tick_params(length=0)
        ax.set_axis_off()
        plt.axhspan(0.5, 0.5,
                    linestyle=plinestyle, edgecolor='black',
                    facecolor='white', linewidth=2)
        filename = 'plinestyle_'+encode(str(plinestyle))+'.png'
        plt.savefig(filename, dpi=dpi2, format='png', bbox_inches=bbox)

        # plt produces blured image on MPL2.0
        if isMPL2:
            im = wx.Image(filename)
            w, h = im.GetSize()
            im.Rescale(w/5, h/5).SaveFile(filename, wx.BITMAP_TYPE_PNG)




from ifigure.mto.py_connection import PyConnection
import numpy as np
import wx
import ifigure.widgets.dialog as dialog
######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.
#      Strong recommendation : make module "independent".
#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module/class
module_name = 'connection_module'
class_name = 'connection_module'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
menu = [("New Connection...", "onNewConnection", True)]
method = ['onNewConnection', 'add_connection']
icon = 'world_link.png'
######## Do not chage this field ####################
can_have_child = True
has_private_owndir = True
######################################################


def add_connection(self, new_name):
    child = PyConnection()
    imodel = self.td.add_child(new_name, child)
    child.setvar('server', new_name)


def onNewConnection(self, e):

    app = self.td.get_app()
    ret, new_name = dialog.textentry(app,
                                     "Enter the name of new connection", "Add Connection", "")
    if ret:
        self.add_connection(new_name)



import multiprocessing as mp
import time


class MPWorker(mp.Process):
    def __init__(self, task_queue, result_queue, *args, **kargs):
        #   def __init__(self,  *args, **kargs):
        super(MPWorker, self).__init__(*args, **kargs)
        self.task_queue = task_queue
        self.result_queue = result_queue

    def run(self, *args, **kargs):
        while True:
            jobs = self.task_queue.get(True)
            if jobs == '1':
                self.result_queue.put('2')
                break
            time.sleep(1)




import piscope
import os
from scipy.io.idl import readsav
from ifigure.utils.vctr_array import expand_vctr3d
from ifigure.interactive import figure, threed, plot, hold


def read_tile_data(list=False):
    dir = os.path.dirname(piscope.__file__)
    dir = os.path.dirname(dir)
    file = os.path.join(dir, 'example', 'cmod_tile_data.sav')
    data = readsav(file)
    return expand_vctr3d(data['xyzseg'], data['nseg'], data['lseg'], list=list)


def plot_tiles():
    '''
    line plot of tile data
    '''
    x, y, z = read_tile_data()
    figure()
    threed('on')
    plot(x, y, z)


def plot_tiles2():
    x, y, z = read_tile_data()
    figure()
    threed('on')
    plot(x, y, z, facecolor=[1, 0, 0, 1])


def plot_tiles3():
    x, y, z = read_tile_data(list=True)
    figure()
    threed('on')
    hold('on')
    for k in range(100):
        m = k+100
        plot(x[m], y[m], z[m], facecolor=[1, 0, 0, 1])



from __future__ import print_function
import wx
from ifigure.utils.cbook import parseStr
from ifigure.mto.py_code import PyData
from ifigure.mto.py_code import PyCode
#from ifigure.add_on.data.mdsplus_tree import MdsplusTreeNode
import wx
import sys
import os
import re
import string
from numpy import *
from collections import OrderedDict
import ifigure.utils.mdsplusr as mds
from ifigure.utils.edit_list import DialogEditList

######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.


#      Strong recommendation : make module "independent".

#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module
module_name = 'mdsplus_tree_node'
class_name = 'mdsplus_tree_node'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
menu = []

#
#   method lists module functions which will
#   be registered as method
#
#   spceical methods:
#      init  : called when mto will be made and
#              this module is first loaded.
#      clean : called when mto will be killed
#
method = ['isMdsNode', '_getitem_',
          'dim_of', 'call_mdsvalue']

icon = 'data.png'
can_have_child = True
######################################################


def isMdsNode(self):
    return True


def _getitem_(self, key):
    txt = self.td.getvar("node")
    print(txt)
    return self.call_mdsvalue(txt)


def dim_of(self, num=0):
    print(self)
    print(self.td)
    node = self.td.getvar("node")
    txt = 'dim_of('+node.strip()+','+str(num)+')'
    return self.call_mdsvalue(txt)


def call_mdsvalue(self, str):
        # parent should be the top of tree
        # such as "cmod", "analysis", "xtomo"

    try:
        parent = self.td
        while parent.get_parent()._obj.isMdsNode():
            parent = parent.get_parent()
    except Exception:
        pass

    tree = parent.getvar("tree")
    shot = parent.getvar("shot")
    server = parent.getvar("server")
    port = parent.getvar("port")
    mds.port = port
    mds.server = server
    print((tree, shot, port, server, str))
    try:
        res = mds.open(tree, shot)
        print(res)
        return mds.value(str)
    except Exception:
        print("!!!!!! Error in evaluating the following node  !!!!!!")
        print(("TREE/SHOT", tree, shot))
        print(("NODE", str))
        print(sys.exc_info())
    return None



"""
    doc string should come here !!!  
    this doc-string will be set to Py.Script.Run() method,
    so that a user can see it from help screen
"""
#     **  Template for a new script  **

#   Following variabs/functions can be used
#    obj : script object
#    top. proj : = obj.get_root_parent()
#    wdir : proj.getvar('wdir')
#    model: model object containing this script
#    param : model param
#    app : proj.app (ifigure application)
#    exit() : exit from script
#    stop() : exit from script due to error
#    write_log(text) : write text to log
#
#    args : parameter arguments passed to Run method (default = ())
#    kwagrs : keyward arguments passed to Run method (default = {})

import numpy as np
from ifigure.interactive import figure


def example():
    import numpy as np
    x = np.linspace(0, 6.28)
    y = np.sin(x)
    return x, y


x, y = example()
v = figure()
v.addpage()
v.plot(x, y)
write_log('making plot')



from ifigure.utils.cbook import parseStr
from ifigure.mto.py_code import PyData
from ifigure.mto.py_code import PyCode
import wx
import os
import re
import string
import logging
import shutil
from numpy import *
from collections import OrderedDict
from ifigure.mto.py_contents import Efitafile

from ifigure.utils.read_afile import read_afile

######################################################
#         Setting for module file for py_module
#   name of module/class
module_name = 'EFIT_Afile'
class_name = 'EFIT_Afile'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#

menu = [("Import File...", "onLoadFile", True),
        ("Update Tree...", "onUpdateTree", True),
        ("+TextEditor...", None,  False),
        ("View Original File", "onOpenOrg", False),
        ("Edit File", "onOpenCurrent", False),
        ("!", None, False)]
method = ['init',
          'txt2tree',
          'onLoadFile',
          'onOpenOrg',  'onUpdateTree']

icon = 'data.png'
can_have_child = False
has_private_owndir = False
######################################################
modename = 'afile_pathmode'
pathname = 'afile_path'
extname = 'afile_ext'


def init(self, *args, **kargs):
    if "src" not in kargs:
        self.onLoadFile(file='')


def load_file(file=None):
    values = read_afile(file)

    nm = Efitafile()
    nm["contents"] = values

    return nm


def load_afile(obj):
    file = obj.path2fullpath(modename=modename,
                             pathname=pathname)

    if file == '':
        return
    for name, child in obj.get_children():
        child.destroy()

    nm = load_file(file)
    obj.setvar0(nm)


def txt2tree(self, filename, txt):
    '''
    save txt to file and load it
    '''
    self_obj = self.td
    if not self_obj.has_owndir():
        self_obj.mk_owndir()

    if os.path.basename(filename) == filename:
        path = os.path.join(self_obj.owndir(), filename)
    else:
        path = filename

    fid = open(path, 'w')
    fid.write(txt)
    fid.close()

    self.td.set_path_pathmode(path, modename, pathname, extname)
    load_afile(self.td)


def onUpdateTree(self, event=None):
    load_afile(self.td)


def onLoadFile(self, e=None, file=''):
    from ifigure.utils.addon_utils import onLoadFile

    if file != '':
        self.td.set_path_pathmode(file, modename, pathname, extname)
        ret = True
    else:
        ret = onLoadFile(self.td, message="Select A-File",
                         modename=modename,
                         pathname=pathname,
                         extname=extname)
    if not ret:
        return False

    load_afile(self.td)


def onOpenOrg(self, e=None):
    from ifigure.utils.addon_utils import onOpenOrg
    onOpenOrg(self.td)


def onOpenCurrent(self, e=None):
    from ifigure.utils.addon_utils import onOpenCurrent
    onOpenCurrent(self.td, modename=modename,
                  pathname=pathname)



from ifigure.mto.fig_obj import FigObj
from ifigure.mto.py_code import PyParam, PyData


class FigobjParam(PyParam, FigObj):
    def generate_artist(self, *args, **kargs):
        pass

    def del_artist(self, *args, **kargs):
        pass


class FigobjData(PyData, FigObj):
    def generate_artist(self, *args, **kargs):
        pass

    def del_artist(self, *args, **kargs):
        pass



from __future__ import print_function
import os
import time
print(" ")
print(" ##########################")
print(" Running Debug Script")
print(" ##########################")
print("")
print(" Running evniroment for script file")
proj = obj.get_root_parent()
print(("   proj    =", proj))
print(("   treeobj =", obj))
print(" ")


print(proj.book1.page6.axes1.spline1._artists[0].get_xdata())



from __future__ import print_function
import wx
import os
import numpy as np
import ifigure
from ifigure.utils.wx3to4 import wxBitmapFromImage, wxEmptyBitmapRGBA

CDS_CHANGED = wx.NewEventType()
EVT_CDS_CHANGED = wx.PyEventBinder(CDS_CHANGED, 1)
CDS_CHANGING = wx.NewEventType()
EVT_CDS_CHANGING = wx.PyEventBinder(CDS_CHANGING, 1)


def window_to_bitmap(window):
    w, h = window.GetClientSize()
    bitmap = wxEmptyBitmapRGBA(w, h)
    wdc = wx.ClientDC(window)
    mdc = wx.MemoryDC(bitmap)
    mdc.Blit(0, 0, w, h, wdc, 0, 0)
    data = np.fromstring(bitmap.ConvertToImage().GetData(), np.uint8)
    print(data.reshape((h, w, 3))[0, 0, :])
    return bitmap


class CustomPanel(wx.Panel):
    def OnMCLost(self, evt):
        evt2 = wx.PyCommandEvent(CDS_CHANGED, wx.ID_ANY)
        evt2.SetEventObject(self)
        self.Unbind(wx.EVT_MOTION)
        self.ReleaseMouse()
        wx.PostEvent(self, evt2)

    def MotionEvent(self, val):
        self._generate_motion_event = val

    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        self.DoDrawing(dc)

    def OnSize(self, evt):
        self.Refresh(eraseBackground=False)

    def Enable(self, value=True):
        wx.Panel.Enable(self, value)
        self.Refresh()

    def Disable(self):
        wx.Panel.Disable(self)
        self.Refresh()


class CustomSingleSlider(CustomPanel):
    def __init__(self, parent, id,  *args, **kargs):
        wx.Panel.__init__(self, parent, id, *args, **kargs)
        self.parent = parent
        self.font = wx.Font(9, wx.FONTFAMILY_DEFAULT,
                            wx.FONTSTYLE_NORMAL,
                            wx.FONTWEIGHT_NORMAL,
                            False, 'Courier 10 Pitch')

        from ifigure.ifigure_config import icondir
        path = os.path.join(icondir, 'image', 'slider.png')

        self._bbmp = wxBitmapFromImage(wx.Image(path, wx.BITMAP_TYPE_PNG))
        self._value = [0.5]
        self._range = [0, 1]
        self._sheight = 6
        self._hmargin = 10
        self._vmargin = 3
        self._bheight = self._vmargin*2 + self._sheight
        self._bwidth = 6

        w, h = self._bbmp.GetSize()
        self._bheight = h
        self._bwidth = w
        self._vmargin = (h - self._sheight)/2
        self._b = None
        self._isDrawn = False
        self._generate_motion_event = False
        self.SetMinSize((w*5, h))

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE,  self.OnSize)
        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnMouseEvent)

    def SetValue(self, value):
        self._value[0] = value
        self.Refresh()

    def GetValue(self):
        return self._value[0]

    def SetRange(self, vrange):
        self._range = vrange
        self.Refresh()

    def GetRange(self):
        return self._range

    def SetSlotHeight(self, h):
        self._sheight = h

    def GetSlotHeight(self, h):
        return self._sheight

    def SetHorizontalMargin(self, h):
        self._hmargin = h

    def GetHorizontalMargin(self):
        return self._hmargin

    def OnMouseEvent(self, evt):
        if not self.IsEnabled():
            return
        if evt.LeftDown():
            x, y = evt.GetPosition()
            w, h = self.GetSize()
            if (abs(y - h/2) > self._sheight):
                return

            w1 = self._calc_w1()
            self._hit = -1
            if abs(x-w1) < self._bwidth:
                self._hit = 0
            if self._hit > -1:
                self.CaptureMouse()
                self.Bind(wx.EVT_MOTION, self.OnMotion)
                self.Bind(wx.EVT_MOUSE_CAPTURE_LOST, self.OnMCLost)
#              self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        elif evt.LeftUp():
            if self.HasCapture():
                evt2 = wx.PyCommandEvent(CDS_CHANGED, wx.ID_ANY)
                evt2.SetEventObject(self)
                self.Unbind(wx.EVT_MOTION)
                self.ReleaseMouse()
                wx.PostEvent(self, evt2)

    def OnMotion(self, evt):
        if not self.IsEnabled():
            return
        bmw, bmh = self._bbmp.GetSize()
        x, y = evt.GetPosition()
        w, h = self.GetSize()

        delta = abs(self._range[1]-self._range[0])
        dx = (w-2*self._hmargin-bmw)

        new_v = float(x - self._hmargin - bmw/2)/dx*delta + self._range[0]
#        if self._hit == 1 and new_v <= self._value[0]: return
        if new_v > self._range[1]:
            return
        if new_v < self._range[0]:
            return
        self._value[self._hit] = new_v
        self.Refresh()

        if self._generate_motion_event:
            evt2 = wx.PyCommandEvent(CDS_CHANGING, wx.ID_ANY)
            evt2.SetEventObject(self)
            wx.PostEvent(self, evt2)

    def DoDrawing(self, dc=None):

        if self.IsEnabled():
            col1 = '#000000'
            col2 = '#FFFFB8'
            col3 = '#ffafaf'
        else:
            col1 = '#7f7f7f'
            col2 = '#FFFFD8'
            col3 = '#ffdfdf'

        self.Unbind(wx.EVT_PAINT)
        if dc is None:
            dc = wx.ClientDC(self)

        w, h = self.GetSize()

        dc.SetFont(self.font)

        yy = int(h/2 - self._sheight/2 - self._vmargin)
        dc.SetDeviceOrigin(0, yy)


        def draw_box(self, w1, dc):
            dc.DrawBitmap(self._bbmp,
                          int(w1-self._bwidth/2),
                          0, True)
            return

        w1 = self._calc_w1()

        dc.SetPen(wx.Pen(col1))
        dc.SetBrush(wx.Brush(col2))
        dc.DrawRoundedRectangle(int(self._hmargin),
                                int(self._vmargin),
                                int(w-2*self._hmargin),
                                int(self._sheight),
                                2.0)
        dc.SetPen(wx.Pen(col1))
        dc.SetBrush(wx.Brush(col3))

        draw_box(self, w1, dc)
#        draw_box(self, w2, dc)

        self._isDrawn = True
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def _calc_w1(self):
        bmw, bmh = self._bbmp.GetSize()
        w, h = self.GetSize()
        delta = abs(self._range[1]-self._range[0])
        w1 = ((w-2*self._hmargin-bmw) / delta * (self._value[0] - self._range[0]) +
              self._hmargin + bmw/2)
        return w1


class CustomDoubleSlider(CustomPanel):
    def __init__(self, parent, id,  *args, **kargs):
        wx.Panel.__init__(self, parent, id, *args, **kargs)
        self.parent = parent
        self.font = wx.Font(9, wx.FONTFAMILY_DEFAULT,
                            wx.FONTSTYLE_NORMAL,
                            wx.FONTWEIGHT_NORMAL,
                            False, 'Courier 10 Pitch')

        from ifigure.ifigure_config import icondir
        path = os.path.join(icondir, 'image', 'slider.png')

        self._bbmp = wxBitmapFromImage(wx.Image(path, wx.BITMAP_TYPE_PNG))
        self._value = [0.1, 0.9]
        self._range = [0, 1]
        self._sheight = 6
        self._hmargin = 10
        self._vmargin = 3
        self._bheight = self._vmargin*2 + self._sheight
        self._bwidth = 6

        w, h = self._bbmp.GetSize()
        self._bheight = h
        self._bwidth = w
        self._vmargin = (h - self._sheight)/2
        self._b = None
        self._isDrawn = False
        self._generate_motion_event = False
        self.SetMinSize((w*5, h))

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE,  self.OnSize)
        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnMouseEvent)

    def SetValue(self, value):
        self._value = value
        self.Refresh()

    def GetValue(self):
        return self._value

    def SetRange(self, vrange):
        self._range = vrange
        self.Refresh()

    def GetRange(self):
        return self._range

    def SetSlotHeight(self, h):
        self._sheight = h

    def GetSlotHeight(self, h):
        return self._sheight

    def SetHorizontalMargin(self, h):
        self._hmargin = h

    def GetHorizontalMargin(self):
        return self._hmargin

    def OnMouseEvent(self, evt):
        if not self.IsEnabled():
            return
        if evt.LeftDown():
            x, y = evt.GetPosition()
            w, h = self.GetSize()
            if (abs(y - h/2) > self._sheight):
                return

            w1, w2 = self._calc_w1_w2()
            self._hit = -1
            if abs(x-w1) < self._bwidth:
                self._hit = 0
            if abs(x-w2) < self._bwidth:
                self._hit = 1
            if self._hit > -1:
                self.Bind(wx.EVT_MOTION, self.OnMotion)
                self.Bind(wx.EVT_MOUSE_CAPTURE_LOST, self.OnMCLost)
                self.CaptureMouse()
#              self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        elif evt.LeftUp():
            if self.HasCapture():
                evt2 = wx.PyCommandEvent(CDS_CHANGED, wx.ID_ANY)
                evt2.SetEventObject(self)
                self.Unbind(wx.EVT_MOTION)
                self.ReleaseMouse()
                wx.PostEvent(self, evt2)
        if evt.RightUp():
            self.SetValue(self._range)
            evt2 = wx.PyCommandEvent(CDS_CHANGED, wx.ID_ANY)
            evt2.SetEventObject(self)
            wx.PostEvent(self, evt2)

    def OnMotion(self, evt):
        if not self.IsEnabled():
            return
        bmw, bmh = self._bbmp.GetSize()
        x, y = evt.GetPosition()
        w, h = self.GetSize()

        delta = abs(self._range[1]-self._range[0])
        dx = (w-2*self._hmargin-bmw)

        new_v = float(x - self._hmargin - bmw/2)/dx*delta + self._range[0]
        if self._hit == 0 and new_v >= self._value[1]:
            return
        if self._hit == 1 and new_v <= self._value[0]:
            return
        if new_v > self._range[1]:
            return
        if new_v < self._range[0]:
            return
        self._value[self._hit] = new_v
        self.Refresh()

        if self._generate_motion_event:
            evt2 = wx.PyCommandEvent(CDS_CHANGING, wx.ID_ANY)
            evt2.SetEventObject(self)
            wx.PostEvent(self, evt2)

    def DoDrawing(self, dc=None):
        if self.IsEnabled():
            col1 = '#000000'
            col2 = '#FFFFB8'
            col3 = '#ffafaf'
        else:
            col1 = '#7f7f7f'
            col2 = '#FFFFD8'
            col3 = '#ffdfdf'

        self.Unbind(wx.EVT_PAINT)
        if dc is None:
            dc = wx.ClientDC(self)

        w, h = self.GetSize()
        dc.SetFont(self.font)

        yy = int(h/2 - self._sheight/2 - self._vmargin)
        dc.SetDeviceOrigin(0, yy)

        def draw_box(self, w1, dc):
            dc.DrawBitmap(self._bbmp,
                          int(w1-self._bwidth/2),
                          0,
                          True)
            return

        w1, w2 = self._calc_w1_w2()

        dc.SetPen(wx.Pen(col1))
        dc.SetBrush(wx.Brush(col2))
        dc.DrawRoundedRectangle(int(self._hmargin),
                                int(self._vmargin),
                                int(w-2*self._hmargin),
                                int(self._sheight),
                                2.)
        dc.SetPen(wx.Pen(col1))
        dc.SetBrush(wx.Brush(col3))
        dc.DrawRectangle(int(w1),
                         int(self._vmargin),
                         int(w2-w1),
                         int(self._sheight))

        draw_box(self, w1, dc)
        draw_box(self, w2, dc)

        self._isDrawn = True
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def _calc_w1_w2(self):
        bmw, bmh = self._bbmp.GetSize()
        w, h = self.GetSize()
        delta = abs(self._range[1]-self._range[0])
        w1 = ((w-2*self._hmargin-bmw) / delta * (self._value[0] - self._range[0]) +
              self._hmargin+bmw/2)
        w2 = ((w-2*self._hmargin-bmw) / delta * (self._value[1] - self._range[0]) +
              self._hmargin+bmw/2)
        return w1, w2


class test_frame(wx.Frame):
    def __init__(self, *args, **kargs):
        wx.Frame.__init__(self, *args, **kargs)
        self.ds = CustomDoubleSlider(self, wx.ID_ANY)
        self.ds.MotionEvent(True)
        self.Layout()
        self.Show()
        self.Bind(EVT_CDS_CHANGED, self.onCDS_CHANGED)
        self.Bind(EVT_CDS_CHANGING, self.onCDS_CHANGING)

    def onCDS_CHANGED(self, evt):
        print(evt.GetEventObject().GetValue())

    def onCDS_CHANGING(self, evt):
        print(evt.GetEventObject().GetValue())


if __name__ == '__main__':
    app = wx.App(False)
    frame = test_frame(None, wx.ID_ANY, "test")
    app.MainLoop()



from __future__ import print_function
import numpy as np
from ifigure.mto.py_contents import Namelist, TSCInputFile
from collections import OrderedDict
import os
######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.
#      Strong recommendation : make module "independent".
#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module/class
module_name = 'tscinput_module'
class_name = 'tscinput_module'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
menu = [("Import File...", "onLoadFile", True),
        ("Export File...", "onWriteFile", False),
        ("Update File", "onUpdateFile", False),
        ("Update Tree", "onUpdateTree", True),
        ("Plot Time Dependent Signals", "onPlotTimeDep", False),
        ("+TextEditor...", None,  False),
        ("View Original", "onOpenOrg", False),
        ("Edit File", "onOpenCurrent", False),
        ("!", None, False)]

method = ['tree2txt', 'onPlotTimeDep', 'onGenerateRestartInput',
          'onLoadFile', 'onWriteFile', 'onOpenOrg',
          'onOpenCurrent', 'onUpdateFile', 'onUpdateTree',
          'init']
icon = 'text.png'
######## Do not chage this field ####################
can_have_child = False
has_private_owndir = False
######################################################
modename = 'namelist_pathmode'
pathname = 'namelist_path'
extname = 'namelist_ext'


def init(self, *args, **kargs):
    if 'src' not in kargs:
        self.onLoadFile()


def split_line(s):
    #form2020= '(8e10.?)'
    d = 10
    ret = []
    for i in range(8):
        if s[d*i:d*(i+1)] != '':
            ret.append(s[d*i:d*(i+1)])
    return ret


def convert_array(arr):
    arr[0] = arr[0][0:2]
    for x in range(len(arr)-1):
        if arr[x+1].strip() == '':
            arr[x+1] = None
        else:
            arr[x+1] = float(arr[x+1])
    return arr


def load_file(file):
    fid = open(file, 'r')
    lines = fid.readlines()
    c = 0
#    nm = Namelist()
    nm = TSCInputFile()

    for line in lines:
        if line.startswith('!'):
            continue
        if line.startswith('c'):
            continue
        line = line.rstrip("\r\n")
        arr = split_line(line)
        try:
            arr = convert_array(arr)
            if arr[0] in nm:
                nm[arr[0]]['data'] = nm[arr[0]]['data'] + [arr[1:], ]
            else:
                d = TSCInputFile()
                d['data'] = [arr[1:], ]
                nm[arr[0]] = d
        except:
            print('conversion error of following line')
            print(line)
            print(arr)
        c = c+1
#        if c>20: break
    fid.close()

    if '11' in nm:
        # for acoef array
        od = nm['11']['data']
        d = TSCInputFile()
        for x in od:
            print(x)
            for i in range(int(x[1])):
                x = x + [0, 0, 0, 0, 0]  # safe gurad...
                label = ('0000'+str(int(x[0]+i)))[-4:]
                d[label] = x[2+i]
        d = TSCInputFile(sorted(list(d.items()), key=lambda t: t[0]))
        nm['11'] = d

    nm = TSCInputFile(sorted(list(nm.items()), key=lambda t: t[0]))
    nm = add_help(nm)
    return nm


def add_help(nm):
    import ifigure
    hfile = os.path.join(os.path.dirname(ifigure.__file__),
                         'add_on', 'model', 'module', 'tscinput_help.txt')
    d = {}
    fid = open(hfile, 'r')
    for line in fid.readlines():
        xy = line.split('\t')
        if len(xy) < 2:
            continue
        x = xy[0]
        y = xy[1]
        d[x[0:2]] = (x[2:].strip(), y)

    for key in nm:
        if key in d:
            nm[key]['name'] = d[key][0]
            nm[key]['format'] = d[key][1]
    nm._help = d
    return nm

#   file='/home/shiraiwa/PycharmProjects/ifigure/sample/genray.in'


def load_tscinputfile(obj):
    file = obj.path2fullpath(modename=modename,
                             pathname=pathname)

    if file == '':
        return

    for name, child in obj.get_children():
        child.destroy()

    nm = load_file(file)

    print(('reading file', file))
    obj.setvar0(nm)

# def fromat_float(num):
#'{: 10.2e}'.format(item)


def tree2txt(self, var0=None):
    def data2lines(key, data):
        lines = []
        for d in data:
            l = key + ' '*(10-len(key))

            for item in d:
                if item is None:
                    l = l + ' '*10
                else:
                    txt = '{: 10.3e}'.format(item)
                    # len(txt)
                    l = l + '{: 10.3e}'.format(item)
            lines.append(l+'\n')
        return lines

    def data2lines_11(key, data):
        lines = []
        for d in data:
            l = str(int(key))
            l = l + ' '*(10-len(l))
            for item in d:
                if item is None:
                    l = l
                else:
                    txt = '{: 10.3e}'.format(item)
                    # len(txt)
                    l = l + txt
            lines.append(l+'\n')
        return lines

    if var0 is None:
        var0 = self.td.getvar0()
    txt = ['c ... title card\n', 'c produced by piscope input generator\n', 'c\n']

    for key0 in list(var0.keys()):
        if key0 == '11':
            for key1 in var0[key0]:
                if key1 == 'name':
                    continue
                if key1 == 'format':
                    continue
                txt.extend(data2lines_11(
                    key0, [[int(key1), 1, var0[key0][key1]]]))
        else:
            txt.extend(data2lines(key0, var0[key0]['data']))

    txt2 = ''.join(txt)
    return txt2


def onGenerateRestartInput(self, e=None, filename='inputa',
                           update_card=None, stop_time=None):
    print(('here', update_card, stop_time))
    var0 = self.td.getvar0()
    if update_card is not None:
        d = TSCInputFile()
        d['00'] = var0['00']
        d['00']['data'][0][0] = 1.0  # restart
        for x in update_card:
            d[x] = var0[x]
        if stop_time is not None:
            d['11'] = TSCInputFile()
            d['11']['0029'] = float(stop_time)
        d['99'] = var0['99']
    txt = self.tree2txt(var0=d)
    filename = os.path.join(self.td.owndir(), filename)
    print(filename)
    self.onWriteFile(e=None, filename=filename, txt=txt)


def onUpdateTree(self, e=None):
    obj = self.td
    load_tscinputfile(self.td)


def onLoadFile(self, e=None, file=''):
    from ifigure.utils.addon_utils import onLoadFile

    if file != '':
        self.td.set_path_pathmode(file, modename, pathname, extname)
        ret = True
    else:
        ret = onLoadFile(self.td, message="Select Namelist File",
                         modename=modename,
                         pathname=pathname,
                         extname=extname)
    if ret:
        load_tscinputfile(self.td)


def onWriteFile(self, e=None, filename='', txt=None):
    from ifigure.utils.addon_utils import onWriteFile

    if txt is None:
        txt = self.tree2txt()
    onWriteFile(self.td, filename=filename,
                txt=txt,
                message='Enter Namelist File Name')


def onUpdateFile(self, e=None, *args, **kargs):
    file = self.td.path2fullpath(modename=modename,
                                 pathname=pathname)
    self.onWriteFile(filename=file)


def onOpenOrg(self, e=None, *args, **kargs):
    from ifigure.utils.addon_utils import onOpenOrg
    onOpenOrg(self.td)


def onOpenCurrent(self, e=None, *args, **kargs):
    from ifigure.utils.addon_utils import onOpenCurrent
    onOpenCurrent(self.td, modename=modename,
                  pathname=pathname)


def onPlotTimeDep(self, e=None, *args, **kargs):
    def plot_data(time, data):
        x = np.array(time, dtype='float').flatten()
        y = np.array(data, dtype='float').flatten()

        x = [t for t in x if not np.isnan(t)]
        y = [t for t in y if not np.isnan(t)]

        return np.array(x), np.array(y)

    help = self.td._var0._help
    data = self.td._var0
    import ifigure.interactive as plt

    plt.figure()
    time = data['18']['data']
    new_plot = True
    for key in help:
        if not key in data:
            continue
        if key == '18':
            continue
        x, y = help[key]
        if y.startswith('-'):
            if not new_plot:
                plt.addpage()
            try:
                xdata, ydata = plot_data(time, data[key]['data'])
                plt.plot(xdata, ydata)
                plt.title(x)
                new_plot = False
            except:
                print(('failed to generate picture for', key))
                new_plot = True
    pass



from matplotlib.figure import Figure
from matplotlib.patches import Polygon
from matplotlib.backends.backend_agg import FigureCanvasAgg
from matplotlib.transforms import Bbox
import matplotlib.patches as patches

import numpy as np


def marker_image(path, size, trans,
                 edgecolor='k', facecolor='b',
                 edgewidth=1.0):

    fig = Figure(figsize=((size*2+1)/72., (size*2+1)/72.), dpi=72)

    # fig.set_edgecolor([1,1,1,0])
    # fig.set_facecolor([1,1,1,0])
    fig.set_edgecolor([0, 0, 0, 0])
    fig.set_facecolor([0, 0, 0, 0])
    fig.patch.set_alpha(0.0)
    fig.clf()
    fig.frameon = False
    ax = fig.add_subplot(111)
    ax.set_position((0, 0, 1, 1))
    ed = ax.transAxes.transform([(0, 0), (1, 1)])

    path = trans.transform_path(path)
    patch = patches.PathPatch(path, facecolor=facecolor, edgecolor=edgecolor,
                              lw=edgewidth)
    ax.patch.set_facecolor([0, 0, 0, 0])
    ax.patch.set_edgecolor([0, 0, 0, 0])
    ax.patch.set_alpha(0.0)
    ax.add_patch(patch)
    ax.set_xlim(-1, 1)
    ax.set_ylim(-1, 1)
    ax.tick_params(length=0)

    ax.set_axis_off()
    canvas = FigureCanvasAgg(fig)
    buff, size = canvas.print_to_buffer()

    im = np.frombuffer(buff, np.uint8).reshape(size[1], size[0], -1)
    #idx = np.where(im[:,:,-1] == 0)
    #im[:,:,0][idx] = 0
    #im[:,:,1][idx] = 0
    #im[:,:,2][idx] = 0
    return im



import bdb
from bdb import Bdb
break_points = {}


class Debugger(Bdb):
    def __init__(self, panel, *args, **kargs):
        Bdb.__init__(self, *args, **kargs)
        self.panel = panel

    def user_call(self, frame, argument_list):
        #       print 'user_call', frame
        self.panel.handle_user_call(frame, argument_list)
#       Bdb.user_call(self, frame, argument_list)

    def user_line(self, frame):
        #       print 'user_line', frame
        self.panel.handle_user_line(frame)
#       return frame
#       Bdb.user_line(self, frame)

    def user_return(self, frame, return_value):
        #       print 'user_return', frame
        self.panel.handle_user_return(frame, return_value)
#       Bdb.user_return(self, frame, return_value)

    def user_exception(self, frame, exc_info):
        #       print 'user_exception', frame
        self.panel.handle_user_exception(frame, exc_info)
#       Bdb.user_exception(self, frame, exc_info)

    def do_clear(self, arg):
        pass
#       print 'do_clear'
#       Bdb.do_clear(self, arg)

    def run(self, *args, **kargs):
        Bdb.run(self, *args, **kargs)


def get_debugger(panel=None):
    d = Debugger(panel)
    for file in break_points:
        for line in break_points[file]:
            d.set_break(d.canonic(file), line)
    return d


def add_breakpoint(file, line):
    #    print file, line
    if file in break_points:
        if not line in break_points[file]:
            break_points[file].append(line)
    else:
        break_points[file] = [line]

#    d = get_debugger()
#    d.set_break(d.canonic(file), line)
#    print d.get_all_breaks()


def rm_breakpoint(file, line):
    if not file in break_points:
        return
    if not line in break_points[file]:
        return
    break_points[file].remove(line)


def has_breakpoint(file, line):
    if not file in break_points:
        return False
    return (line in break_points[file])


def get_breakpoint(file):
    if not file in break_points:
        return []
    return break_points[file]



import wx
import sys
import weakref
import threading
import traceback
import wx.aui as aui


class RedirectOutput(object):
    def __init__(self, stdout, stderr):

        self.stdout = stdout
        self.stderr = stderr
        self.redirect_list = []
        # store normal stdout/stderr
        self.sys_stdout = sys.stdout
        self.sys_stderr = sys.stderr

    def turn_on(self):
        if self.sys_stdout != self.stdout:
            sys.stdout = self
            sys.stderr = self

    def turn_force_on(self):
        sys.stdout = self
        sys.stderr = self

    def turn_off(self):
        sys.stdout = self.sys_stdout
        sys.stderr = self.sys_stderr

    def write(self, string):
        try:
            self.do_write(string)
        except:
            self.turn_off()
            traceback.print_exc()
            self.turn_on()

    def _safe_write(self, string):
        if hasattr(self.stdout, 'write'):
            self.stdout.write(string)
        else:
            self.sys_stdout.write(string)

    def do_write(self, string):
        t = threading.current_thread()
        for x in self.redirect_list:
            if x[0] == t:
                   ##Do not put a print statement here!!##
                wx.CallAfter(x[1], string)
                if t.name == 'MainThread':
                    try:
                        self._safe_write(string)
                    except IOError:  # may fail to write to stdout on macapplet
                        raise
                return
                # if string != "\n":
                #wx.MessageBox(string, "Error!")
        if t.name == 'MainThread':
            try:
                self._safe_write(string)
            except IOError:  # may fail to write to stdout on macapplet
                raise
        else:
            if string == '\n':
                wx.CallAfter(self._safe_write, string)
            else:
                wx.CallAfter(self._safe_write, 'Thread ('+t.name + '):'+string)

    def add(self, t, method):
        self.redirect_list.append((t, method))

    def rm(self, t):
        self.redirect_list = [x for x
                              in self.redirect_list if x[0] != t]

    def flush(self):
        if hasattr(self.stdout, 'flush'):
            self.stdout.flush()



from __future__ import print_function
#
#   ArgsParser
#
#   argument parser
#
#   usage :
#
#   def hogehoge(*args, **kywds):
#      '''
#      hogehoge(x,  w=None)
#      hogehoge(x, y, w=None, o=xxx...)
#      '''
#
#      p = ArgsParser()
#      p.add_var('x', check)   ### mandatory argument
#      p.add_opt('y', 1, check)  ### optional argument
#      p.add_key('w', None) ### keyword argment
#
#      p.set_pair('x', 'y') ### require x and y need to be given togehter
#      p.set_pair('n', 'w') ### require n and w are not given togehter
#
#      p.set_ndconvert("x") ### convert x to ndarray if possible
#                           ### x should be iterable for this conversion
#      check : type check
#         'ndarray': check if it is ndarray
#         'iter' : check if it is array
#         'str'  : check if it is string
#         'int'  : check if it is int
#         'float'  : check if it is float
#         'nonstr'  : check if it is not str
#         'real'  : check if it is either float or int
#         'bool'  : check if it is bool
#         'dynamic'  : check if it is dynamic expression
#         'sequence'  : check if it is sequence
#         'number':  numbers (int, long, float, complex)
#         'numbers': sequence of numbers
#         'empty' : check if it is []
#
#      check can be list and conncted by '|'
#         ['float', 'int'] :
#          True if it is number ( = same as ['real'])
#         ['iter|nonstr', 'dynamic'] :
#          True if it is either 'dynamic' or 'iterable and not string'
#
#      v, kywds, flag = p.process(*args, **kywds)
#
#      v is a dict v[name] = value
#      kywds is remaining keywords which is not processed
#      flag is True if rule is successfully applied
#
from ifigure.utils.cbook import isiterable, isndarray, isdynamic, issequence, isnumber
import six
import numpy as np
import ifigure.utils.debug as debug
dprint1, dprint2, dprint3 = debug.init_dprints('ArgsParser')


def bit(n, l):
    if n == 0:
        return [0]*l
    v = [0]*l
    k = 0
    while n >= 2**k:
        v[k] = int((n & 2**k) != 0)
        k = k+1
    return v


def convert_ndarray(v, name):
    if name not in v:
        return v
    if v[name] is None:
        return v
    if isdynamic(v[name]):
        return v
    if isiterable(v[name]) and not isndarray(v[name]):
        try:
            v[name] = np.array(v[name])
        except:
            return v
    if not isiterable(v[name]):
        try:
            v[name] = np.array([v[name]])
        except:
            return v
    if np.iscomplex(v[name]).any():
        return v
    if np.iscomplexobj(v[name]):
        v[name] = v[name].real
    else:
        v[name] = v[name].astype(np.float64)
    return v


def apply_squeeze(v, name, minimum_1D=False):
    if name not in v:
        return v
    if isdynamic(v[name]):
        return v
    try:
        if (v[name] is not None and not minimum_1D):
            v[name] = np.squeeze(v[name])
        if (v[name] is not None and minimum_1D and
                v[name].dim > 1):
            v[name] = np.squeeze(v[name])
    except:
        pass
    return v


class ArgsParser(object):
    def __init__(self):
        self.vars = []
        self.key = []
        self.exclusives = []
        self.pairs = []
        self.ndconvert = []
        self.squeeze = []
        self.squeeze1D = []
        self.defv_list = []

    def add_var(self, name, t=None):
        self.vars.append((name, t))

    def add_opt(self, name, defv, t=None):
        self.vars.append((name, defv, t))

    def add_key(self, key, defv, t=None):
        self.key.append((key, defv, t))

    def set_default_list(self, defv_list):
        self.defv_list = defv_list

    def add_key2(self, key, t=None):
        if isinstance(key, tuple):
            for k in key:
                self.key.append((k, self.defv_list[k], None))
        else:
            self.key.append((key, self.defv_list[key], t))

    def set_exclusive(self, name1, name2):
        self.exclusives.append((name1, name2))

    def set_pair(self, name1, name2):
        self.pairs.append((name1, name2))

    def set_ndconvert(self, *args):
        for n in args:
            self.ndconvert.append(n)

    def set_squeeze(self, *args):
        for n in args:
            self.squeeze.append(n)

    def set_squeeze_minimum_1D(self, *args):
        for n in args:
            self.squeeze1D.append(n)

    def check_pairs(self, value):
        flag = True
        for name1, name2 in self.pairs:
            if (name1 not in value and
                    name2 in value):
                flag = False
            if (name1 in value and
                    name2 not in value):
                flag = False
        return flag

    def check_exclusives(self, value):
        flag = True
        for name1, name2 in self.exclusives:
            if (name1 in value and
                    name2 in value):
                flag = False
            if (name1 in value and
                    name2 in value):
                flag = False
        return flag

    def has_exclusives(self, name, value):
        for name1, name2 in self.exclusives:
            if name1 == name:
                return name2 in value
            if name2 == name:
                return name1 in value
        return False

    def check(self, value, incond):
        def do_check(value, cond):
            if cond == 'ndarray':
                return isndarray(value)
            if cond == 'can_ndreal_array':
                try:
                    void = np.array(value).astype(np.float64)
                    return void.size > 0
                except:
                    pass
                return False
            elif cond == 'iter':
                return isiterable(value)
            elif cond == 'sequence':
                return issequence(value)
            elif cond == 'str':
                if six.PY2:
                    return isinstance(value, str) or isinstance(value, unicode)
                else:
                    return isinstance(value, str)
            elif cond == 'nonstr':
                if six.PY2:                
                    return not (isinstance(value, str) or isinstance(value, unicode))
                else:
                    return not isinstance(value, str)
            elif cond == 'int':
                return isinstance(value, int)
            elif cond == 'float':
                return isinstance(value, float)
            elif cond == 'real':
                return isinstance(value, float) or isinstance(value, int)
            elif cond == 'dynamic':
                return isdynamic(value)
            elif cond == 'bool':
                return isinstance(value, bool)
            elif cond == 'number':
                return isnumber(value)
            elif cond == 'empty':
                if (isinstance(value, list) and
                        len(value) == 0):
                    return True
                return False
            elif cond == 'numbers':
                if issequence(value):
                    if len(value) == 0:
                        return False
                    return isnumber(value[0])
                else:
                    return False
            elif cond == 'any':
                return True
            print(('ArgsParser::Unknown condition (ignored)', cond))
            return True

        def do_check2(value, conds):
            ret = True
            cond = conds.split('|')
            for c in cond:
                ret = ret and do_check(value, c)
            return ret

        if isiterable(incond) and not isinstance(incond, str):
            a = False
            for c in incond:
                a = a or do_check2(value, c)
            return a
        else:
            return do_check(value, incond)

    def process(self, *args, **kywds):
        success = True
        value = {}
        flag = True
        cases = [bit(i, len(self.vars)) for i in range(2**len(self.vars))]
        
        for case in cases:
            # check length of arguments
            if sum(case) != len(args):
                continue

            # check required field selection
            k = 0
            flag = True
            for v in self.vars:
                if len(v) == 2 and case[k] == 0:
                    flag = False
                k = k+1
            if not flag:
                continue

            k1 = 0
            k2 = 0
            flag = True
            value = {}
            for v in self.vars:
                if case[k1] == 1:
                    if not self.check(args[k2], v[-1]):
                        flag = False
                        break
                    value[v[0]] = args[k2]
                    k2 = k2 + 1
                k1 = k1+1
            # check pairs
            if not self.check_pairs(value):
                flag = False
            if not self.check_exclusives(value):
                flag = False
            if flag:
                break

        if not flag:
            return {}, kywds, [], False

        # add optional using default value
        # add only parameter which counterpart of exclusive
        ### pair is not set
        defv_names = []
        for v in self.vars:
            if (len(v) == 3 and v[0] not in value and
                    not self.has_exclusives(v[0], value)):
                value[v[0]] = v[1]
                defv_names.append(v[0])

        # handle keyword values
        for key, v, t in self.key:
            if key in kywds:
                #               value.append((key, kywds[key]))
                val = kywds[key]
                if t is not None:
                    if not self.check(val, t):
                        dprint1('keyword type error : ', key)
                        return {}, kywds, [], False
                value[key] = val
                del kywds[key]
            else:
                value[key] = v
                defv_names.append(key)
#               value.append((key, v))

        for name in self.ndconvert:
            value = convert_ndarray(value, name)

        for name in self.squeeze:
            value = apply_squeeze(value, name)
        for name in self.squeeze1D:
            value = apply_squeeze(value, name, minimum_1D=True)
        return value, kywds, defv_names, success


if __name__ == '__main__':
    def hogehoge1(*args, **kywds):
        '''
        hogehoge(x,  w=None)
        hogehoge(x, y, w=None, o=xxx...)
        '''

        p = ArgsParser()
        p.add_var('x')  # mandatory argument
        p.add_opt('y', 'default y')  # optional argument
        p.add_key('w', 'default keyword')  # keyword argment

        v, kywds, defv_names,  flag = p.process(*args, **kywds)
        print((v, kywds, defv_names, flag))

    def hogehoge2(*args, **kywds):
        '''
        hogehoge(y, s  w=None)
        hogehoge(x, y, s, w=None, o=xxx...)

        same as plot
        '''
        p = ArgsParser()
        p.add_opt('x', None, ['iter|nonstr', 'dynamic'])  # optional argument
        p.add_var('y', ['iter|nonstr', 'dynamic'])  # mandatory argument
        p.add_opt('s', 'default s', 'str')  # optional argument
        p.add_key('w', 'default keyword')  # keyword argment
        p.set_ndconvert('x', 'y')
        p.set_squeeze_minimum_1D('x', 'y')
#      p.set_squeeze('x', 'y')

        v, kywds, d, flag = p.process(*args, **kywds)
        print((v, kywds, d, flag))

    def hogehoge3(*args, **kywds):
        '''
        hogehoge(z)
        hogehoge(x, y, z)
        hogehoge(z, n)
        hogehoge(z, v)
        hogehoge(x, y, z, n)
        hogehoge(x, y, z, v)

        same as contour
        '''
        p = ArgsParser()
        p.add_opt('x', None, ['iter'])  # optional argument
        p.add_opt('y', None, ['iter'])  # optional argument
        p.add_var('z', ['iter'])  # mandatory argument
        p.add_opt('n', None, ['int', 'float'])  # optional argument
        p.add_opt('v', None, ['iter'])  # optional argument
        p.set_pair('x', 'y')
        p.set_exclusive('n', 'v')

        v, kywds, d, flag = p.process(*args, **kywds)
        print((v, kywds, d, flag))

    print("hogehoge1(x, y(option), w='default keyword)")
    print('case 1')
    print("   hogehoge1([1,3,5], 'new y')")
    hogehoge1([1, 3, 5], 'new y')
    print('')
    print('case 2')
    print("   hogehoge1([1,3,5], w='new keyword', other='???')")
    hogehoge1([1, 3, 5], w='new keyword', other='???')
    print('')
    print('case 3 (shoud have flag = False)')
    print("   hogehoge( w='new keyword')")
    hogehoge1(w='new keyword')

    print("hogehoge2(x(option), y, s(option), w='default keyword)")
    hogehoge2([1, 4, 9])
    hogehoge2([1, 2, 3], [1, 4, 9])
    hogehoge2([1, 4, 9], '=s')
    hogehoge2([1, 4, 9], 's')
    hogehoge2([1, 2, 3], [1, 4, 9], 's')
    hogehoge2([1, 2, 3], [1, 4, 9], 1)
    hogehoge2([1], [1])

    print("hogehoge3(x(option), y(option), z, n(option), v(option))")
    import numpy as np
    z = np.arange(25).reshape(5, 5)
    x = np.arange(5)
    y = np.arange(5)
    n = 10
    v = np.arange(10)
    hogehoge3(z)
    hogehoge3(x, y, z)
    hogehoge3(z, n)
    hogehoge3(z, v)



'''
   decorator which make sure that call happens in 
   the main thread.
'''
from functools import wraps
import wx
from six.moves import queue as Queue
import threading


class CallError(object):
    pass


def at_wxthread(func):
    @wraps(func)
    def checker(*args, **kargs):
        def func2(callable, queue, *args, **kargs):
            try:
                v = callable(*args, **kargs)
                queue.put(v)
            except:
                queue.put(CallError())

        t = threading.current_thread()
        if t.name == 'MainThread':
            return func(*args, **kargs)
        else:
            q = Queue.Queue()
            wx.CallAfter(func2, func,  q, *args, **kargs)
            value = q.get()
            if isinstance(value, CallError):
                raise ValueError('Asyc call failed')
        return value
    return checker



from __future__ import print_function

'''
  read gfile

     2015 09 22
        Sign of Bt and Ip is positive when it orients
        to anti-clockwise direction when viewing the tokamak
        from top to bottom.

        gfile cpasma and bcnter is negative when they
        are clockwise (viewing from top)
        IOW, gfile uses the R, phi, Z cylindrical coordinate system.
       
        In gfile, psi is written as if it is always 
        increase from the center to the edge. For positive
        current (in the R, phi, Z cylindrical coordinate system),
        psi should DECREASE as it goes outward to have a 
        correct sign of Br, Bz. This module, flipps the sign 
        of psirz, ffprim, and pprime when Ip in g-file header
        is >0. Note that safety factor is still always positive.

     2015 10 20
        some gfile can have a different mesh size for fpol, q and
        others. in this case, the first line of file has the third
        mesh size number

'''

from ifigure.utils.cbook import parseStr
from ifigure.mto.py_code import PyData
from ifigure.mto.py_code import PyCode
import wx
import os
import re
import string
import logging
import shutil
from numpy import *
from collections import OrderedDict
from ifigure.utils.edit_list import DialogEditList
from ifigure.mto.py_contents import Efitgfile
import numpy as np
######################################################
#         Setting for module file for py_module
#   name of module/class
module_name = 'EFIT_Gfile'
class_name = 'EFIT_Gfile'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#

menu = [("Import File...", "onLoadFile", True),
        ("Export File...", "onWriteFile", True),
        ("Update File...", "onUpdateFile", True),
        ("Update Tree...", "onUpdateTree", True),
        ("Plot Equilibrium", "onPlotEq", True),
        ("Plot MidPlane", "onPlotMid", True),
        ("+Modify...", None,  False),
        ("Scale Bt...", "onScaleBt",  False),
        ("Scale Ip...", "onScaleIp",  False),
        ("!", None, False),
        ("+TextEditor...", None,  False),
        ("View Original File", "onOpenOrg", False),
        ("Edit File", "onOpenCurrent", False),
        ("!", None, False)]
method = ['scale_b', 'scale_i', 'scale_p', 'init',
          'txt2tree', 'tree2txt',
          'onLoadFile', 'onWriteFile', 'onPlotEq',
          'onOpenOrg', 'onOpenCurrent', 'onScaleBt', 'onScaleIp',
          'onPlotMid', 'onUpdateFile', 'onUpdateTree']

icon = 'data.png'
can_have_child = False
has_private_owndir = False
######################################################
modename = 'gfile_pathmode'
pathname = 'gfile_path'
extname = 'gfile_ext'


def string_split_ig(s, sep):
    return [x for x in re.split(sep, s) if x != '']

    '''
    print("here", s, sep, t)
    for i in reversed(range(len(t))):
        if t[i] == '':
            del t[i]
    return t
    '''


def eval_form2020(s):
    # form2020= '(5e16.9)'
    d = 16
    ret = []
    for i in range(5):
        #     print s[d*i:d*(i+1)],s[d*i:d*(i+1)].strip().__repr__()
        if s[d*i:d*(i+1)].strip() != '':
            ret.append(float(s[d*i:d*(i+1)]))
    return ret


def readandstrip(f):
    line = ''
    count = 0
    while line == '':
        line = f.readline()
        line = line.rstrip("\r\n")
        count = count + 1
        if count > 5:
            return ''

    return line


def make_form2020(array):
    line0 = []
    line = ''
    i = 0
    while i < len(array):
        line = line+'{: 13.9e}'.format(array[i])
        i = i+1
        if (i % 5) == 0:
            line0.append(line)
            line = ''

    if len(line) != 0:
        line0.append(line)

    return line0


def load_array_form2020(f, size):
    k = 0
    arr = []
    while (k < size):
        arr = arr+eval_form2020(readandstrip(f))
        k = k+5
    return np.array(arr)


def load_matrix_form2020(f, size1, size2):
    k = 0
    arr = []
    while k < size1*size2:
        arr = arr+eval_form2020(readandstrip(f))
        k = k+5
    return np.array(arr).reshape(size1, size2)


def add_extra_data(val):
    fpol = val['fpol']
    isPlasma = val['isPlasma']
    psirz = val['psirz']
    ssimag = val['ssimag']
    ssibdry = val['ssibdry']
    ssibry = val['ssibry']

    xpsi = (val['psirz']-val['ssimag'])/(val['ssibdry']-val['ssimag'])
    for m, xv in enumerate(val['rgrid']):
        for n, yv in enumerate(val['zgrid']):
            if not isPlasma[n, m]:
                xpsi[n, m] = 1.0
    rgrid = val['rgrid']
    zgrid = val['zgrid']
    xx, yy = np.meshgrid(rgrid, zgrid)

    xxx = np.linspace(0, 1, fpol.shape[0])
    fp = np.interp(xpsi.flatten(), xxx, val['fpol'])
    fp = fp.reshape(xx.shape)
    fc = np.interp(xpsi.flatten(), xxx, val['ffprim'])
    fc = fc.reshape(xx.shape)
    pr = np.interp(xpsi.flatten(), xxx, val['pres'])
    pr = pr.reshape(xx.shape)
    pc = np.interp(xpsi.flatten(), xxx, val['pprime'])
#   this is to check if pprime is computed correctly in ACCOME
#   psimesh = np.linspace(ssimag, ssibry, len(val['pres']))
#   pc = np.interp(xpsi.flatten(), xxx,
#                 np.gradient(val['pres'])/np.gradient(psimesh))
    pc = pc.reshape(xx.shape)
    qr = np.interp(xpsi.flatten(), xxx, val['qpsi'])
    qr = qr.reshape(xx.shape)

    pr[isPlasma != True] = 0.0
    pc[isPlasma != True] = 0.0
    fc[isPlasma != True] = 0.0
    val["pressrz"] = pr
    val["qrz"] = qr
    val["btrz"] = fp/xx

    dpsidz, dpsidr = np.gradient(psirz)
    brrz = -dpsidz/(zgrid[1]-zgrid[0])/xx
    bzrz = dpsidr/(rgrid[1]-rgrid[0])/xx
    val["brrz"] = brrz
    val["bzrz"] = bzrz

    mu0 = 4e-7*3.1415926535
    val["jtrz"] = (xx*pc+fc/xx/mu0)/1e6  # 1e6=(MA/m2)

    k = (val["zmaxis"] - rgrid[0])/(rgrid[1] - rgrid[0])

    from ifigure.utils.interpolate import interp2d

    f = interp2d(rgrid, zgrid, psirz, kind='cubic')
    val['npsimid'] = np.array([(f(r, val["zmaxis"]) - ssimag)/(ssibry - ssimag)
                               for r in rgrid]).flatten()
    f1 = interp2d(rgrid, zgrid, val["btrz"], kind='cubic')
    f2 = interp2d(rgrid, zgrid, val["bzrz"], kind='cubic')
    val['gammamid'] = np.array([np.arctan(f2(r, val["zmaxis"])
                                          / f1(r, val["zmaxis"]))*180/3.1415926
                                for r in rgrid]).flatten()
    val['bzmid'] = np.array([f2(r, val["zmaxis"])
                             for r in rgrid]).flatten()
    val['btmid'] = np.array([f1(r, val["zmaxis"])
                             for r in rgrid]).flatten()
    f = interp2d(rgrid, zgrid, val["jtrz"], kind='cubic')
    val['jtmid'] = np.array([f(r, val["zmaxis"])
                             for r in rgrid]).flatten()
    f = interp2d(rgrid, zgrid, val["pressrz"], kind='cubic')
    val['pressmid'] = np.array([f(r, val["zmaxis"])
                                for r in rgrid]).flatten()
    f = interp2d(rgrid, zgrid, val["qrz"], kind='cubic')
    val['qmid'] = np.array([f(r, val["zmaxis"])
                            for r in rgrid]).flatten()


def load_file(file=None):
    def is_ascii(s):
        return all([ord(c) < 128 for c in s])
        '''
        try:
            s.decode('ascii')
        except:
            return False
        return True
        '''
#   file='/home/shiraiwa/g1101019014.01340'
    f = open(file, 'r')

    line = f.readline()
    line = line.rstrip("\r\n")
    tmp = string_split_ig(line[49:], ' |,')

    header = line[:48]
    idum = int(tmp[0])
    mw = int(tmp[1])
    mh = int(tmp[2])
    mw2 = mw
    if len(tmp) == 4:
        try:
            mw2 = int(tmp[3])
            if mw2 == 0:
                mw2 = mw
        except:
            pass

    xdim, zdim, rzero, rgrid1, zmid = eval_form2020(readandstrip(f))
    rmaxis, zmaxis, ssimag, ssibdry, bcentr = eval_form2020(readandstrip(f))
    cpasma, ssimag, xdum, rmaxis, xdum = eval_form2020(readandstrip(f))
    zmaxis, xdum, ssibry, xdum, xdum = eval_form2020(readandstrip(f))

    fpol = load_array_form2020(f, mw2)
    pres = load_array_form2020(f, mw2)
    ffprim = load_array_form2020(f, mw2)
    pprime = load_array_form2020(f, mw2)

    psirz = load_matrix_form2020(f, mh, mw)
    qpsi = load_array_form2020(f, mw2)

    try:
        nbbbs, limitr = string_split_ig(readandstrip(f), ' |,')
    # print nbbbs, limitr
        nbbbs = int(nbbbs)
        limitr = int(limitr)
        if nbbbs != 0:
            rzbbbs = load_matrix_form2020(f, nbbbs, 2)

        xylim = load_matrix_form2020(f, limitr, 2)
    except:
        limitr = 0
        nbbbs = 0
        rzbbbs = np.zeros((2, 1))
        xylim = np.zeros((2, 1))

    rgrid = rgrid1 + xdim*np.arange(mw)/(mw-1.)
    zgrid = zmid-0.5*zdim + zdim*np.arange(mh)/(mh-1.)

    nm = Efitgfile()

    psirzraw = psirz.copy()
    if cpasma > 0:
        sss = -1
        ssimag = ssimag*sss
        ssibdry = ssibdry*sss
        ssibry = ssibry*sss
        psirz = psirz*sss
        ffprim = ffprim*sss
        pprime = pprime*sss

    nm["table"] = {}
    val = nm["table"]
    val["header"] = header
    val["idum"] = idum
    val["mw"] = mw
    val["mh"] = mh
    val["xdim"] = xdim
    val["zdim"] = zdim
    val["rzero"] = rzero
    val["rgrid1"] = rgrid1
    val["zmid"] = zmid
    val["rmaxis"] = rmaxis
    val["zmaxis"] = zmaxis
    val["ssimag"] = ssimag
    val["ssibdry"] = ssibdry
    val["bcentr"] = bcentr
    val["cpasma"] = cpasma
    val["ssibry"] = ssibry

    val["rgrid"] = rgrid
    val["zgrid"] = zgrid
    val["psirz"] = psirz
    val["psirzraw"] = psirzraw
    val["fpol"] = fpol
    val["pres"] = pres
    val["ffprim"] = ffprim
    val["pprime"] = pprime
    val["qpsi"] = qpsi
    val["nbbbs"] = nbbbs
    if nbbbs > 0:
        val["rbbbs"] = rzbbbs[:, 0]
        val["zbbbs"] = rzbbbs[:, 1]
    else:
        from ifigure.extra.efit_tools import find_psi_contour
        rzbbbs = find_psi_contour(rgrid, zgrid, psirz, rmaxis, zmaxis,
                                  ssibry, return_all=False)

        nbbbs = rzbbbs.shape[0]
        val["nbbbs"] = nbbbs
        val["rbbbs"] = rzbbbs[:, 0]
        val["zbbbs"] = rzbbbs[:, 1]
    val["nlim"] = limitr
    if limitr > 0:
        val["xlim"] = xylim[:, 0]
        val["ylim"] = xylim[:, 1]
    else:
        val["xlim"] = []
        val["ylim"] = []
    # else:
    #   return nm
    # namelist section
    sec = None
    end_flag = False

    while 1:
        line0 = f.readline()
        if not is_ascii(line0):
            continue
        if not line0:
            break
        line0 = line0.rstrip("\r\n")
        line0 = ' '.join(line0.split())
        if line0.endswith('/'):
            line = line0[:-1]
        else:
            line = line0

        if line == '':
            continue
        if line.startswith('&'):
            s = string_split_ig(line, ' |,')
            sec = s[0][1:]
            print('making new sec ', sec, line.__repr__(), s)
            nm[sec] = OrderedDict()
            if len(s) > 1:
                line = ' '.join(s[1:])
            else:
                continue
        if sec is None:
            continue  # skip unitl first &*** starts
        sall = string_split_ig(line, ' |,')

        i = 0
        while i < len(sall):
            if sall[i].find('=') != -1:
                if sall[i] == '=':    # '='
                    sall[i-1] = sall[i-1]+'='
                    del sall[i]
                    continue
                if sall[i].startswith('='):  # '=value'
                    sall[i-1] = sall[i-1]+'='
                    sall[i] = sall[i][1:]

                    continue
                if sall[i].endswith('='):
                    i = i+1
                    continue  # 'name='
                k = sall[i].split('=')
                sall[i] = k[0]+'='
                sall.insert(i+1, k[1])
            i = i+1

        for s in sall:
            if s.find('=') != -1:
                k = s.split('=')
                varname = k[0]
#          s[0]=k[1]
#          print s
#          if s[0] is '': del s[0]
                if debug != 0:
                    print(('create dict key', sec, varname))
                # print 'create dict key', sec, varname
                nm[sec][varname] = []
                if s.endswith('/'):
                    sec = None
                continue
        # for lines without 'xxxx = '
            # print s, sec, varname
            nm[sec][varname] = nm[sec][varname]+parseStr(s)
        if line0.endswith('/'):
            sec = None

    f.close()

    xx, yy = np.meshgrid(rgrid, zgrid)
#   isPlasma = xx.copy()
    sss = len(rgrid)*len(zgrid)
    isPlasma = np.array([False]*sss).reshape(len(zgrid), len(rgrid))
    for m, xv in enumerate(rgrid):
        for n, yv in enumerate(zgrid):
            dx = rzbbbs[:, 0] - xv
            dy = rzbbbs[:, 1] - yv
            d1 = np.sqrt(dx[:-1]**2 + dy[:-1]**2)
            d2 = np.sqrt(dx[1:]**2 + dy[1:]**2)

            d = (dx[:-1]*dy[1:] - dx[1:]*dy[:-1])/d1/d2
            d = d[np.abs(d) < 1.0]
            xxxx = sum(np.arcsin(d))
            isPlasma[n, m] = (np.abs(xxxx) > 3)
#              print isPlasma[n, m] > 3
#   val['isPlasma0'] = isPlasma
#   print 'here'
#   isPlasma =  np.abs(isPlasma) > 3
    val["isPlasma"] = isPlasma

    try:
        add_extra_data(val)
    except:
        print("can't evaulate extra info")

    return nm
#   return val, nm,  header

# def add_gfile_2_tree(self_obj):


def scale_p(self, factor=None):

    td = self.td
    if factor is None:
        list = [["", "Enter P Scale Factor", 2],
                ["P sclae", str(1), 0]]

        flag, value = DialogEditList(list)
        if flag:
            factor = float(value[1])
        else:
            return

    print(('Pres scale by', str(factor)))
    td[:]["table"]["pres"] *= factor
    td[:]["table"]["pressmid"] *= np.abs(factor)
    td[:]["table"]["pressrz"] *= factor
    td[:]["table"]["pprime"] *= factor
    add_extra_data(td[:]["table"])
    print('Updating g-file')
    self.onUpdateFile()


def scale_b(self, factor=None):

    td = self.td
    if factor is None:
        list = [["", "Enter Bt Scale Factor", 2],
                ["Bt sclae", str(1), 0]]

        flag, value = DialogEditList(list)
        if flag:
            factor = float(value[1])
        else:
            return

    print(('Bt scale by', str(factor)))

    td[:]["table"]["ffprim"] *= factor*factor
    td[:]["table"]["fpol"] *= factor
    td[:]["table"]["qpsi"] *= np.abs(factor)
    td[:]["table"]["bcentr"] *= factor
    add_extra_data(td[:]["table"])
    print('Updating g-file')
    self.onUpdateFile()


def scale_i(self, factor=None):

    td = self.td
    if factor is None:
        list = [["", "Enter Ip Scale Factor", 2],
                ["Ip sclae", str(1), 0]]

        flag, value = DialogEditList(list)
        if flag:
            factor = float(value[1])
        else:
            return

    print(('Ip scale by', str(factor)))

    td[:]["table"]["psirz"] *= factor
    td[:]["table"]["qpsi"] *= np.abs(1./factor)
    td[:]["table"]["cpasma"] *= factor
    td[:]["table"]["ssibry"] *= factor
    td[:]["table"]["ssibdry"] *= factor
    td[:]["table"]["ssimag"] *= factor
    add_extra_data(td[:]["table"])
    print('Updating g-file')
    self.onUpdateFile()


def onScaleIp(self, evt):
    self.scale_i()


def onScaleBt(self, evt):
    self.scale_b()


def init(self, *args, **kargs):
    if "src" not in kargs:
        self.onLoadFile(file='')


def load_gfile(obj):
    file = obj.path2fullpath(modename=modename,
                             pathname=pathname)

    if file == '':
        return
    for name, child in obj.get_children():
        child.destroy()

    nm = load_file(file)
    obj.setvar0(nm)


def txt2tree(self, filename, txt):
    '''
    save txt to file and load it
    '''
    if not self.td.has_owndir():
        self.td.mk_owndir()

    if os.path.basename(filename) == filename:
        path = os.path.join(self.td.owndir(), filename)
    else:
        path = filename

    fid = open(path, 'w')
    fid.write(txt)
    fid.close()

    self.td.set_path_pathmode(path, modename, pathname, extname)
    load_gfile(self.td)


def tree2txt(self, filename=''):
    '''
    make tree to txt and update owndir file
    '''

    self_obj = self.td

    txt = ['']

    table = self_obj[:]["table"]

    header = table["header"]
    idum = table["idum"]
    mh = table["mh"]
    mw = table["mw"]

    l0 = table["header"] + \
        '{: 4d}'.format(idum)+'{: 4d}'.format(mw)+'{: 4d}'.format(mh)

    xdum = 0
    if table["cpasma"] > 0:
        sss = -1
    else:
        sss = 1

    l1 = make_form2020([table["xdim"],
                        table["zdim"],
                        table["rzero"],
                        table["rgrid1"],
                        table["zmid"]])

    l2 = make_form2020([table["rmaxis"],
                        table["zmaxis"],
                        table["ssimag"]*sss,
                        table["ssibdry"]*sss,
                        table["bcentr"]])

    l3 = make_form2020([table["cpasma"],
                        table["ssimag"]*sss,
                        xdum,
                        table["rmaxis"],
                        xdum])

    l4 = make_form2020([table["zmaxis"],
                        xdum,
                        table["ssibry"]*sss,
                        xdum,
                        xdum])

    txt.append(l0+"\n")
    txt.append(l1[0]+"\n")
    txt.append(l2[0]+"\n")
    txt.append(l3[0]+"\n")
    txt.append(l4[0]+"\n")

    for field in ["fpol", "pres", "ffprim", "pprime", "psirz", "qpsi"]:
        value = table[field]
        if field in ['psirz', 'ffprim', 'pprime']:
            value = value*sss
        for line in make_form2020(value.flatten()):
            txt.append(line+"\n")

    nbbbs = table["nbbbs"]
    nlim = table["nlim"]
    l5 = '{: 5d}'.format(nbbbs)+'{: 5d}'.format(nlim)

    txt.append(l5+"\n")

    r = table["rbbbs"]
    z = table["zbbbs"]
    c = concatenate((r, z))
    rzbbbs = c.reshape(2, len(r)).transpose().flatten()
    for line in make_form2020(rzbbbs):
        txt.append(line+"\n")
    r = table["xlim"]
    z = table["ylim"]
    c = concatenate((r, z))
    xylim = c.reshape(2, len(r)).transpose().flatten()
    for line in make_form2020(xylim):
        txt.append(line+"\n")

    txt2 = ''.join(txt)
    return txt2


def onUpdateTree(self, event=None):
    load_gfile(self.td)


def onLoadFile(self, e=None, file=''):
    from ifigure.utils.addon_utils import onLoadFile

    if file != '':
        self.td.set_path_pathmode(file, modename, pathname, extname)
        ret = True
    else:
        cwd = os.getcwd()
        if self.td.has_owndir():
            os.chdir(self.td.owndir())
        try:
            ret = onLoadFile(self.td, message="Select G-File",
                             modename=modename,
                             pathname=pathname,
                             extname=extname)
        except:
            import traceback
            traceback.print_exc()
            pass
        os.chdir(cwd)

    if not ret:
        return False

    load_gfile(self.td)


def onWriteFile(self, e=None, filename=''):
    from ifigure.utils.addon_utils import onWriteFile

    txt = self.tree2txt()
    onWriteFile(self.td, filename=filename,
                txt=txt,
                message='Enter G-file Name',
                wildcard='G-file(g*.*)|g*.*|Any|*')


def onUpdateFile(self, event=None):
    file = self.td.path2fullpath(modename=modename,
                                 pathname=pathname)
    self.onWriteFile(filename=file)


def onOpenOrg(self, *args, **kargs):
    from ifigure.utils.addon_utils import onOpenOrg
    onOpenOrg(self.td)


def onOpenCurrent(self, *args, **kargs):
    from ifigure.utils.addon_utils import onOpenCurrent
    onOpenCurrent(self.td, modename=modename,
                  pathname=pathname)


def onPlotMid(self, e=None):
    self_obj = self.td

    if self_obj.getvar0() is None:
        return
    proj = self_obj.get_root_parent()
    from ifigure.interactive import figure

    v = figure()
    v.nsec(1, 5)

    path = self_obj.get_full_path()
    x = self_obj[:]["table"]["rgrid"]
    y = self_obj[:]["table"]["zgrid"]
    psi = self_obj[:]["table"]["psirz"]
    ssimag = self_obj[:]["table"]["ssimag"]
    ssibdry = self_obj[:]["table"]["ssibdry"]

    fpol = self_obj[:]["table"]["fpol"]
    pprime = self_obj[:]["table"]["pprime"]
    ffprim = self_obj[:]["table"]["ffprim"]
    pres = self_obj[:]["table"]["pres"]
    qpsi = self_obj[:]["table"]["qpsi"]

    zmaxis = self_obj[:]["table"]["zmaxis"]

    btmid = self_obj[:]["table"]["btmid"]
    # brmid = self_obj[:]["table"]["brmid"]
    bzmid = self_obj[:]["table"]["bzmid"]
    jtmid = self_obj[:]["table"]["jtmid"]
    pressmid = self_obj[:]["table"]["pressmid"]
    qmid = self_obj[:]["table"]["qmid"]

    nr = psi.shape[1]
    dr = x[1]-x[0]
    dz = y[1]-y[0]
#    br = psi[:][j+1]
    j = psi.shape[0]//2
    psi_mid = psi[:][j]
    psi_mid = array([interp(zmaxis, y, psi[:, i]) for i in range(nr)])

    v.update(False)
    v.isec(0)
    obj = v.plot(x, bzmid)
    v.title(r"$B_{\mathrm{tor, z}}\/\mathrm{(T)}$")
    obj._name = 'bz'
    # obj=FigPlot(x,bzmid)
    # axes.add_child('bz', obj)
    obj = v.plot(x, btmid)
    obj._name = 'bt'
#    obj=FigPlot(x,sqrt(bt_mid*bt_mid+br_mid*br_mid+bz_mid*bz_mid))
#    axes.add_child('ball', obj)

    v.isec(1)
    v.title(r"$J_{\mathrm{tor}}\/\mathrm{(MA/m^2)}$")
    obj = v.plot(x, jtmid)
    obj._name = 'jtor'

    v.isec(2)
    v.title(r"$P\/\mathrm{(MPa)}$")
    obj = v.plot(x, pressmid/1e6)
    obj._name = 'pres'

    v.isec(3)
    v.title(r"$\mathrm{safety\,factor}$")
    obj = v.plot(x, qmid)
    obj._name = 'qpsi'

    v.isec(4)
    v.title("pitch angle")
    obj = v.plot(x, -np.arctan2(bzmid, btmid)*180/np.pi)
    obj._name = 'gamma'

    v.update(True)
#    app=self_obj.get_root_parent().app

#    app.show_page(0)


def onPlotEq(self, e=None, viewer=None):
    self_obj = self.td
    if self_obj.getvar0() is None:
        return

    path = self_obj.get_full_path()
    x = self_obj.get_contents("table", "rgrid")
    y = self_obj.get_contents("table", "zgrid")
    z = self_obj.get_contents("table", "psirz")
    try:
        xlim = self_obj.get_contents("table", "xlim")
        ylim = self_obj.get_contents("table", "ylim")
        rb = self_obj.get_contents("table", "rbbbs")
        zb = self_obj.get_contents("table", "zbbbs")
        fpol = self_obj.get_contents("table", "fpol")
        ssimag = self_obj.get_contents("table", "ssimag")
        ssibdry = self_obj.get_contents("table", "ssibdry")
        btrz = self_obj.get_contents("table", "btrz")
        brrz = self_obj.get_contents("table", "brrz")
        bzrz = self_obj.get_contents("table", "bzrz")
        jtrz = self_obj.get_contents("table", "jtrz")
        bnorm = sqrt(btrz**2+brrz**2+bzrz**2)
        bp = sqrt(brrz**2+bzrz**2)
        bangle = np.arctan2(bp, bnorm)
#       isPlasma = self_obj.get_contents("table", "isPlasma")
        no_lim_b = False
    except:
        no_lim_b = True

    rmaxis = self_obj.get_contents("table", "rmaxis")
    zmaxis = self_obj.get_contents("table", "zmaxis")

    if viewer is None:
        from ifigure.interactive import figure
        plt = figure()
    else:
        plt = viewer
        plt.cls()
    plt.update(False)

    plt.subplot(4, 3, [0, 1, 2, 3], [4, 5, 6, 7])
    plt.isec(4)
    plt.image(x, y, z, alpha=0.5)
    plt.contour(x, y, z, 30)
    if eval('self_obj[:]["table"]["nlim"]') != 0:
        plt.plot(xlim, ylim)
    if eval('self_obj[:]["table"]["nbbbs"]') != 0:
        plt.plot(rb, zb)
    plt.xlabel('R')
    plt.ylabel('Z')
    plt.title('PSI')

    if not no_lim_b:
        plt.isec(5)
        plt.title('B-norm')
        plt.image(x, y, bnorm, alpha=0.5)
        plt.contour(x, y, bnorm, 30)
        if self_obj[:]["table"]["nlim"] > 0:
            plt.plot(xlim, ylim)
        if self_obj[:]["table"]["nbbbs"] > 0:
            plt.plot(rb, zb)
        plt.xlabel('R')
        plt.ylabel('Z')

    plt.isec(0)
    pres = self_obj.get_contents("table", "pres")

    plt.plot(pres)
    plt.title('pressure')
    plt.xlabel('flux index')

    plt.isec(1)
    qpsi = self_obj.get_contents("table", "qpsi")
    plt.plot(qpsi)
    plt.title('qpsi')
    plt.xlabel('flux index')

    plt.isec(2)
    fpol = self_obj.get_contents("table", "fpol")
    plt.plot(fpol)
    plt.title('f_pol')
    plt.xlabel('flux index')
    plt.update(True)

    if no_lim_b:
        return

    from ifigure.extra.efit_tools import flux_average, npsi2psi, npsi2npsit

    npsis = np.linspace(0.02, 0.99, 20)
    psis = [npsi2psi(self_obj, npsi) for npsi in npsis]
    sqrt_npsit = [np.sqrt(npsi2npsit(self_obj, npsi)) for npsi in npsis]
    jtor = [flux_average(x, y, z, rmaxis, zmaxis, jtrz, psi)
            for psi in psis]
    plt.isec(3)
    plt.plot(sqrt_npsit, jtor)
    plt.title('j_tor')
    plt.xlabel('sqrt_toroidal_flux')
    plt.update(True)



from __future__ import print_function
import numpy as np

import matplotlib.artist as artist
from matplotlib.artist import Artist, allow_rasterization
from matplotlib.figure import Figure

from operator import itemgetter
from ifigure.utils.cbook import EraseBitMap


class FigureMod(Figure):
    #    def __init__(self, *args, **kargs):
    #        self._frameDrown = False
    #        Figure.__init__(self,*args, **kargs)

    def _call_draw(self, aa, draw, renderer):
        bk = aa.get_rasterized()
        aa.set_rasterized(self.get_rasterized())
        draw(renderer)
        aa.set_rasterized(bk)

    def draw_others(self, renderer, dsu=None):

        #if self.frameon: self.patch.draw(renderer)
        # a list of (zorder, func_to_call, list_of_args)
        if dsu is None:
            dsu = []
        for a in self.patches + self.lines + self.artists:
            # skip if a is  frameart
            if hasattr(a, '_is_frameart') and a._is_frameart:
                continue
            dsu.append((a.get_zorder(), a, a.draw, renderer))

        # override the renderer default if self.suppressComposite
        # is not None
        not_composite = renderer.option_image_nocomposite()
        if self.suppressComposite is not None:
            not_composite = self.suppressComposite

        if len(self.images) <= 1 or not_composite or \
                not allequal([im.origin for im in self.images]):
            for a in self.images:
                dsu.append((a.get_zorder(), a, a.draw, renderer))
        else:
            # make a composite image blending alpha
            # list of (_image.Image, ox, oy)
            mag = renderer.get_image_magnification()
            ims = [(im.make_image(mag), im.ox, im.oy)
                   for im in self.images]

            im = _image.from_images(self.bbox.height * mag,
                                    self.bbox.width * mag,
                                    ims)

            im.is_grayscale = False
            l, b, w, h = self.bbox.bounds

            def draw_composite(_a):
                gc = renderer.new_gc()
                gc.set_clip_rectangle(self.bbox)
                gc.set_clip_path(self.get_clip_path())
                renderer.draw_image(gc, l, b, im)
                gc.restore()

            dsu.append((self.images[0].get_zorder(),
                        self, draw_composite, None))

        # render the figure text
        # first apply default to suptitle
        if self.figobj._title_artist is not None:
            tfont = self.figobj.getp('title_font')
            tweight = self.figobj.getp('title_weight')
            tstyle = self.figobj.getp('title_style')
            tsize = self.figobj.getp('title_size')
            tinfo = self.figobj.getp('suptitle_labelinfo')
            title = self.figobj._title_artist
            if tinfo[2] == 'default':
                title.set_family(tfont)
            if tinfo[3] == 'default':
                title.set_weight(tweight)
            if tinfo[4] == 'default':
                title.set_style(tstyle)
            if tinfo[5] == 'default':
                title.set_size(tsize)
        for a in self.texts:
            dsu.append((a.get_zorder(), a, a.draw, renderer))

        for a in self.legends:
            dsu.append((a.get_zorder(), a, a.draw, renderer))

        dsu.sort(key=itemgetter(0))
        for _zorder, a, draw_func, arg in dsu:
            self._call_draw(a, draw_func, arg)

    def draw_frame(self, renderer):
        # draw the figure bounding box, perhaps none for white figure
        #        self.patch.set_facecolor((0.75, 0.75, 0.75, 0.1))
        if self.frameon:
            self.patch.draw(renderer)
            dsu = []
            for a in self.patches + self.lines + self.artists:
                if hasattr(a, '_is_frameart') and a._is_frameart:
                    dsu.append((a.get_zorder(), a, a.draw, renderer))
            dsu.sort(key=itemgetter(0))
            for _zorder, a, draw_func, arg in dsu:
                self._call_draw(a, draw_func, arg)

    def draw_axes(self, renderer, axes, noframe=False):
        def walk_children(a):
            yield a
            for a2 in a.get_children():
                for a3 in walk_children(a2):
                    yield a3

        if self.frameon and not noframe:
            self._call_draw(self.patch, self.patch.draw, renderer)

        #
        #  this is show axis box front ....
        #
        alist = [a for a in walk_children(axes)
                 if hasattr(a, 'figobj') and a.figobj is not None]
        normalz = [a.get_zorder() for a in alist
                   if not a.figobj._floating]
        if len(normalz) != 0:
            z = max(normalz)
            for a in alist:
                if (hasattr(a, 'nozsort') and
                        a.get_zorder() < z):
                    a.set_zorder(z+0.001)

        dsize = self.figobj.getp('ticklabel_size')
        dsize2 = self.figobj.getp('axeslabel_size')
        tickfont = self.figobj.getp('tick_font')
        tickweight = self.figobj.getp('tick_weight')
        tickstyle = self.figobj.getp('tick_style')

        if not hasattr(axes, 'zaxis'):
            var = ((axes.figobj.get_axis_param('x'),
                    axes.get_xaxis(),
                    'x'),
                   (axes.figobj.get_axis_param('y'),
                    axes.get_yaxis(),
                    'y'))
        else:
            var = ((axes.figobj.get_axis_param('x'),
                    axes.get_xaxis(),
                    'x'),
                   (axes.figobj.get_axis_param('y'),
                    axes.get_yaxis(),
                    'y'),
                   (axes.figobj.get_axis_param('z'),
                    axes.zaxis,
                    'z'))

        xticks = axes.xaxis.get_major_ticks() + axes.xaxis.get_minor_ticks()
        yticks = axes.yaxis.get_major_ticks() + axes.yaxis.get_minor_ticks()
        for tick in xticks+yticks:
            tick.label1.set_family(tickfont)
            tick.label1.set_style(tickstyle)
            tick.label1.set_weight(tickweight)
            tick.label2.set_family(tickfont)
            tick.label2.set_style(tickstyle)
            tick.label2.set_weight(tickweight)

        for p, axis, name in var:
            if p.lsize == 'default':
                axes.tick_params(axis=name, labelsize=dsize)
            offsetText = axis.get_offset_text()

            if p.labelinfo[2] == 'default':
                #                 print dsize2
                axis.label.set_family(tickfont)
            if p.labelinfo[3] == 'default':
                axis.label.set_weight(tickweight)
#                 print dsize2
            if p.labelinfo[4] == 'default':
                #                 print dsize2
                axis.label.set_style(tickstyle)
            if p.labelinfo[5] == 'default':
                #                 print dsize2
                axis.label.set_size(dsize2)
            if p.otsize == 'default':
                offsetText.set_size(dsize)
            offsetText.set_weight(tickweight)
            offsetText.set_family(tickfont)
            offsetText.set_style(tickstyle)

#                func().label.set_size(dsize2)
        tfont = self.figobj.getp('title_font')
        tweight = self.figobj.getp('title_weight')
        tstyle = self.figobj.getp('title_style')
        tsize = self.figobj.getp('title_size')
        tinfo = axes.figobj.getp('title_labelinfo')
        title = axes.title
#        if axes.figobj.getp('use_def_size')[0]:
#
#            title.set_size(dsize)
        if tinfo[2] == 'default':
            title.set_family(tfont)
        if tinfo[3] == 'default':
            title.set_weight(tweight)
        if tinfo[4] == 'default':
            title.set_style(tstyle)
        if tinfo[5] == 'default':
            title.set_size(tsize)

        w = self.figobj.getp('axesbox_width')
        for s in axes.spines:
            axes.spines[s].set_linewidth(w)
        w = self.figobj.getp('axestick_width')
        axes.tick_params(axis='both', width=float(w))

        self._call_draw(axes, axes.draw, renderer)

    def draw_from_bitmap(self, renderer):

        if not self.get_visible():
            return
        renderer.open_group('figure')
        # do nothing
        renderer.close_group('figure')
        #self._cachedRenderer = renderer
        renderer = self.canvas.get_renderer()
        #self.canvas.draw_event(renderer)

        from matplotlib.backend_bases import DrawEvent
        self.canvas.callbacks.process("draw_event",
                                      DrawEvent("draw_from_bitmap", self.canvas,
                                                renderer))

    def draw(self, renderer):
        """
        Render the figure using :class:`matplotlib.backend_bases.RendererBase` instance renderer
        """
#        print('calling figure::draw')
#        import traceback
#        traceback.print_stack()
        return super(FigureMod, self).draw(renderer)



#
#   CSV (comma separated values) file
#       as of today, it supports a file in which
#          the first line is key
#          all columns has the same lenght
import numpy as np
import ifigure
import wx
import os
from ifigure.utils.edit_list import DialogEditList
######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.
#      Strong recommendation : make module "independent".
#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module/class
module_name = 'csv_module'
class_name = 'csv_module'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
menu = [("Import File...",   "onLoadFile", True),
        ("Export File...", "onWriteFile", True),
        ("Update Tree", "onUpdateTree", True),
        ("Convert to Float...", "onConvFloat", True),
        ("Convert to Integer...", "onConvInt", True),
        ("Convert to String...", "onConvStr", True)]

method = ['onLoadFile', 'onUpdateTree', 'onWriteFile',
          'onConvFloat', 'onConvStr', 'onConvInt',
          'ask_field',
          'init', 'init_after_load']

icon = 'data.png'
######## Do not chage this field ####################
can_have_child = True
has_private_owndir = True
######################################################

modename = 'csv_pathmode'
pathname = 'csv_path'
extname = 'csv_ext'
wildcard = 'csv(*.csv)|*.csv|Any|*'


def init(self, *args, **kargs):
    #   a function called when py_module is initialized
    self.td.mk_owndir()
    if 'src' not in kargs:
        self.onLoadFile()


def load_csv_file(obj):
    file = obj.path2fullpath(modename=modename,
                             pathname=pathname)

    if file == '':
        return
    # clean first

    for name, child in obj.get_children():
        child.destroy()

    import csv

    with open(file, 'rU') as csvfile:
        spamreader = csv.reader(csvfile, dialect='excel')
#                                delimiter=' ', quotechar='|')
        keys = next(spamreader)
        d = {k: [] for k in keys}
        for row in spamreader:
            for k, num in zip(keys, row):
                d[k].append(num)

    from ifigure.mto.py_code import PyData

    data = PyData()
    obj.add_child('data', data)
    for k in d:
        data.setvar(k, d[k])
#    nm = load_file(file)
#    if nm is None:
#         obj.setvar0(None)
#         return
#    obj.setvar0(d)


def export_csvfile(obj, filename):
    import csv
    var = obj.data.getvar()
    names = list(var.keys())
    with open(filename, 'w', newline='') as csvfile:
        spamwriter = csv.writer(csvfile, delimiter=',',
                                quotechar='|')# quoting=csv.QUOTE_MINIMAL)
        print(names)
        spamwriter.writerow(names)
        for k in range(len(var[names[0]])):
            data = [var[n][k] for n in names]
            spamwriter.writerow(data)


def onWriteFile(self, e=None):
    from ifigure.utils.addon_utils import onWriteFile
    app = wx.GetApp().TopWindow
    obj = self.td
    file = obj.get_root_parent().getvar('filename')
    if file is None:
        file = obj.path2fullpath(modename=modename,
                                 pathname=pathname)
    open_dlg = wx.FileDialog(app,
                             message='Save .csv file',
                             defaultDir=os.path.dirname(file),
                             defaultFile='untitled.csv',
                             style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
                             wildcard=wildcard)

    if open_dlg.ShowModal() != wx.ID_OK:
        open_dlg.Destroy()
        return
    else:
        filename = str(open_dlg.GetPath())
        open_dlg.Destroy()
    export_csvfile(obj, filename)


def onUpdateTree(self, e=None):
    obj = self.td
    load_csv_file(obj)


def onLoadFile(self, e=None, file=''):
    from ifigure.utils.addon_utils import onLoadFile

    if file != '':
        self.td.set_path_pathmode(file, modename, pathname, extname)
        ret = True
    else:
        ret = onLoadFile(self.td, message="Select CSV File",
                         modename=modename,
                         pathname=pathname,
                         extname=extname,
                         wildcard='csv(*.csv)|*.csv|Any|*',
                         ask_org_copy=True)
    if ret:
        load_csv_file(self.td)
        ifigure.events.SendChangedEvent(self.td)


def init_after_load(self, olist, nlist):
    obj = self.td
    # load_csv_file(obj)


def ask_field(self, dest):
    vars = self.td.data.getvar()
    list6 = [["", "Select field to convert", 2],
             [None, None, 36, {'col': 4, 'labels': list(vars.keys())}], ]
    value = DialogEditList(list6, modal=True,
                           style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER,
                           tip=None,
                           title="Convert to "+dest,
                           parent=self.td.get_app())
    checked = [x for x, b in value[1][1] if b]
    return value[0], checked


def onConvStr(self, evt):
    def conv(data, name):
        d = [str(x) for x in data.eval(name)]
        data.setvar(name, np.array(d))
    value, checked = self.ask_field('String')
    if value:
        for x in checked:
            conv(self.td.data, str(x))


def onConvFloat(self, evt):
    def conv(data, name):
        d = [float(x) for x in data.eval(name)]
        data.setvar(name, np.array(d).astype(float))
    value, checked = self.ask_field('Float')
    if value:
        for x in checked:
            conv(self.td.data, str(x))


def onConvInt(self, evt):
    def conv(data, name):
        d = [float(x) for x in data.eval(name)]
        data.setvar(name, np.array(d).astype(int))
    value, checked = self.ask_field('Integer')
    if value:
        for x in checked:
            conv(self.td.data, str(x))



from __future__ import print_function
import numpy as np
import matplotlib


from matplotlib.widgets import Button
import matplotlib.gridspec as gridspec


class color(object):
    def __init__(self, fig):
        self.color = ''
        self.collist = ['blue', 'green', 'red',
                        'cyan', 'magenta', 'yellow',
                        'black', 'white']
        self.r = 2
        self.c = 5
        self.fig = fig

    def build_interface(self, spec):
        gs0 = gridspec.GridSpecFromSubplotSpec(
            self.r, self.c,
            subplot_spec=spec)

        k = 0
        for i in range(1, self.c):
            for j in range(0, self.r):
                ax = self.fig.add_subplot(gs0[j, i])
                bt = Button(ax, self.collist[k])
                bt.on_clicked(self.hndl_event)
                k = k+1

    def get_size(self):
        pass

    def get_value(self):
        pass

    def set_value(self):
        pass

    def hndl_event(self, event):
        exit
        print(event)


if __name__ == '__main__':
    import matplotlib.pyplot as plt

    fig = plt.figure()
    pl = color(fig)

    gs = gridspec.GridSpec(1, 1, left=0, bottom=0, right=1, top=0.2)
    subplot_spec = gs.new_subplotspec([0, 0])
    pl.build_interface(subplot_spec)

    plt.show()



import six

import matplotlib as mpl
from matplotlib.axes import Axes
from matplotlib.ticker import ScalarFormatter
from mpl_toolkits.mplot3d.axes3d import Axes3D
import mpl_toolkits.mplot3d.proj3d as proj3d
import mpl_toolkits.mplot3d.art3d as art3d
import matplotlib.transforms as trans
import ifigure.events as events
from matplotlib.artist import allow_rasterization
import numpy as np
import traceback
from matplotlib.collections import Collection, LineCollection, \
    PolyCollection, PatchCollection, PathCollection


class AxesMod(Axes):
    def __new__(cls, *args, **kargs):
        if isinstance(args[0], Axes):
            args[0].__class__ = AxesMod
            return args[0]
        else:
            return Axes.__new__(cls)

    def __init__(self, *args, **kargs):
        self._nomargin_mode = False
        self._offset_trans_changed = False
        self._ypad = None
        if isinstance(args[0], Axes):
            return
        super(AxesMod, self).__init__(*args, **kargs)

    def set_nomargin_mode(self, mode):
        self._nomargin_mode = mode
        if True:
            ay = self.get_yaxis()
            if len(ay.get_major_ticks()) > 0:
                yt = ay.get_major_ticks()[0]
                self._ypad = yt.get_pad()

    def get_nomargin_mode(self):
        return self._nomargin_mode

    def cz_plot(self, x, y, z, **kywds):
        from ifigure.matplotlib_mod.cz_linecollection import CZLineCollection
        a = CZLineCollection(x, y, z,  **kywds)
        self.add_collection(a)
        return a

    def _get_all_ax_artists(self, inframe=False):
        '''
        return all ax artists in the container
        '''
        artists = []

        artists.extend(self.collections)
        artists.extend(self.patches)
        artists.extend(self.lines)
        artists.extend(self.texts)
        artists.extend(self.artists)

        # the frame draws the edges around the axes patch -- we
        # decouple these so the patch can be in the background and the
        # frame in the foreground. Do this before drawing the axis
        # objects so that the spine has the opportunity to update them.
        if self.axison and self._frameon:
            artists.extend(six.itervalues(self.spines))

        if self.axison and not inframe:
            if self._axisbelow:
                self.xaxis.set_zorder(0.5)
                self.yaxis.set_zorder(0.5)
            else:
                self.xaxis.set_zorder(2.5)
                self.yaxis.set_zorder(2.5)
            artists.extend([self.xaxis, self.yaxis])
        if not inframe:
            artists.append(self.title)
            artists.append(self._left_title)
            artists.append(self._right_title)
        artists.extend(self.tables)
        if self.legend_ is not None:
            artists.append(self.legend_)

        return artists

    @allow_rasterization
    def draw(self, renderer):
        ax = self.get_xaxis()
        ay = self.get_yaxis()

        formatters = [self.get_xaxis().get_major_formatter(),
                      self.get_xaxis().get_minor_formatter(),
                      self.get_yaxis().get_major_formatter(),
                      self.get_yaxis().get_minor_formatter()]
        if hasattr(self, 'get_zaxis'):
            formatters.extend([self.get_zaxis().get_major_formatter(),
                               self.get_zaxis().get_minor_formatter()])

        #
        # 2015.10 ScalarFormatter stores usetex, useMathtext when the
        #         object is instantiated. To switch usetex to print
        #         text as text object in pdf, those interanal values
        #         needs to be manipulated.

        usetex = mpl.rcParams['text.usetex']
        org_useMathtext = []
        org_usetex = []
        for f in formatters:
            if isinstance(f, ScalarFormatter):
                org_useMathtext.append(f._useMathText)
                org_usetex.append(f._usetex)
                if usetex:
                    f._useMathText = False
                    f._usetex = True
            else:
                org_usetex.append(False)
                org_useMathtext.append(False)  # dummy data

        ### text size is not know here ###
        t = ay.get_offset_text()
        w, h, d = renderer.get_text_width_height_descent('lp',
                                                         t._fontproperties,
                                                         ismath=False)
        wx, hx, d = renderer.get_text_width_height_descent('lp',
                                                           ax.label._fontproperties,
                                                           ismath=False)
        wy, hy, d = renderer.get_text_width_height_descent('lp',
                                                           ay.label._fontproperties,
                                                           ismath=False)
        xticks = ax.get_major_ticks() + ax.get_minor_ticks()
        yticks = ay.get_major_ticks() + ay.get_minor_ticks()
        if self._nomargin_mode:
            for xtick in xticks:
                xtick.set_pad(-abs(xtick.get_pad()))
                xtick.label1.set_verticalalignment('bottom')
                xtick.label2.set_verticalalignment('top')
            for ytick in yticks:
                ytick.set_pad(- wy)
                ytick.label1.set_horizontalalignment('left')
                ytick.label2.set_horizontalalignment('right')
            self.title.set_position((0.5, 0.9))
            self.title.set_verticalalignment('top')

            if not self._offset_trans_changed:
                t2x = trans.ScaledTranslation(0, 3*w/72,
                                              self.figure.dpi_scale_trans)
                xtext = ax.get_offset_text()
                t0 = xtext.get_transform()
                xtext.set_transform(t0+t2x)
                px = xtext.get_position()
                xtext.set_position((0.9, px[1]))
                xtext = ax.label
                t0 = xtext.get_transform()
                xtext.set_transform(t0+t2x)

                t2y = trans.ScaledTranslation(0, -2*w/72,
                                              self.figure.dpi_scale_trans)
                ytext = ay.get_offset_text()
                t0 = ytext.get_transform()
                ytext.set_transform(t0+t2y)
                py = ytext.get_position()
                ytext.set_position((0.1, py[1]))
                self._offset_trans_changed = True

                t2y2 = trans.ScaledTranslation(3*h/72, 0,
                                               self.figure.dpi_scale_trans)
                ytext = ay.label
                t0 = ytext.get_transform()
                ytext.set_transform(t0+t2y2)
        else:
            for xtick in xticks:
                xtick.set_pad(abs(xtick.get_pad()))
                xtick.label1.set_verticalalignment('top')
                xtick.label2.set_verticalalignment('bottom')
            for ytick in yticks:
                if self._ypad is None:
                    self._ypad = 4
                ytick.set_pad(self._ypad)
                ytick.label1.set_horizontalalignment('right')
                ytick.label2.set_horizontalalignment('left')
            self.title.set_position((0.5, 1.0))
            self.title.set_verticalalignment('baseline')

            if self._offset_trans_changed:
                t2x = trans.ScaledTranslation(0, -3*w/72,
                                              self.figure.dpi_scale_trans)
                xtext = ax.get_offset_text()
                t0 = xtext.get_transform()
                xtext.set_transform(t0+t2x)
                px = xtext.get_position()
                xtext.set_position((1.0, px[1]))
                xtext = ax.label
                t0 = xtext.get_transform()
                xtext.set_transform(t0+t2x)

                t2y = trans.ScaledTranslation(0, 2*w/72,
                                              self.figure.dpi_scale_trans)
                ytext = ay.get_offset_text()
                t0 = ytext.get_transform()
                ytext.set_transform(t0+t2y)
                py = ytext.get_position()
                ytext.set_position((0.0, py[1]))
                self._offset_trans_changed = False

                t2y2 = trans.ScaledTranslation(-3*h/72, 0,
                                               self.figure.dpi_scale_trans)
                ytext = ay.label
                t0 = ytext.get_transform()
                ytext.set_transform(t0+t2y2)

        area = self.figobj.getp('area')
        edge_only = self.figobj._edge_only

        if edge_only[1]:
            hide_xbottom = True
            col1 = xticks[0].label1.get_color()
            for xtick in xticks:
                xtick.label1.set_color('None')
        else:
            hide_xbottom = False
        if edge_only[0]:
            hide_yleft = True
            col2 = yticks[0].label1.get_color()
            for ytick in yticks:
                ytick.label1.set_color('None')
        else:
            hide_yleft = False

        for a in self.artists:
            if not hasattr(a, 'figobj'):
                continue
            if a.figobj.get_namebase() == 'legend':
                l = a.figobj
                l.set_legendlabel_prop(l.getp('legendlabelprop'), a)

        #
        #  zorder of rasterized artsits is moved to negative
        #  before rendering
        #
        artists = self.get_children()
        artists.remove(self.patch)
        artists.remove(self.title)
        artists.remove(self._left_title)
        artists.remove(self._right_title)
        rastered_a = [a for a in artists if a.get_rasterized()]

        if len(rastered_a) > 0:
            delta_a = np.max([a.zorder for a in rastered_a])+1
            for a in rastered_a:
                a.zorder = a.zorder-delta_a

        try:
            super(AxesMod, self).draw(renderer)
        except:
            traceback.print_exc()
        finally:
            if len(rastered_a) > 0:
                for a in rastered_a:
                    a.zorder = a.zorder+delta_a

            if hide_xbottom:
                for xtick in xticks:
                    xtick.label1.set_color(col1)
            if hide_yleft:
                for ytick in yticks:
                    ytick.label1.set_color(col2)

            #
            # Restore orinal setting of usetex in scallar formatter
            #
            for org1, org2, f in zip(org_useMathtext, org_usetex, formatters):
                if isinstance(f, ScalarFormatter):
                    f._useMathText = org1
                    f._usetex = org2
